/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.app.view.navigation.NewFolderDialog");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.Tree");
dojo.require("dojox.data.dom");
dojo.require("dojo.data.ItemFileReadStore");

(function() {

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;

dojo.declare("com.ibm.team.reports.web.app.view.navigation.NewFolderDialog", [dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/navigation/templates/NewFolderDialog.html"),
	widgetsInTemplate: true,
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");
		this.ok = messages.ok;
		this.cancel = messages.cancel;
		this._parentFolder = args.parentFolder;
		this._shared = args.shared;
		this._controller = args.controller;
	},
	
	postCreate: function() {
		this.dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: this.contentNode,
			modal: true,
			primaryTitle: messages.newFolder,
			width: "350px",
			isClosable: true
		});
		this.connect(this._folderName, "onkeyup", this._onType);
		this.connect(this._okButton, "onclick", this._onOk);
		this.connect(this._cancelButton, "onclick", this._onCancel);
		var srh = new ServiceResponseHandler(this, "_setTeamAreas", "_handleError");
		ProcessClient.getProcessAreaHierarchies(srh);
	},
	
	showErrorMessage: function(msg) {
		this.messageArea.className = "error";
		dojox.data.dom.removeChildren(this.messageArea);
		this.messageArea.appendChild(document.createTextNode(msg));
	},
	
	close: function() {
		this.dialog.close();
	},
	
	_createTree: function(dataStore) {
		var treeStore = new dojo.data.ItemFileReadStore({
			data: dataStore
		});
		var tree = new dijit.Tree({
			store: treeStore,
			widgetId: 'processAreaHierarchyTree'
		});
		this._tree = tree;
		this._treeNode.appendChild(tree.domNode);
		
		this.connect(tree, "focusNode", this._onFocus);
		this.connect(tree, "_onEnterKey", this._onOk);
		this.connect(tree.domNode, "ondblclick", this._onOk);
	},
	
	_setTeamAreas: function(processAreaHierarchy) {
		this._projectAreaSet = this._createProjectAreaSet(processAreaHierarchy);
		var store = this._createDataStore(processAreaHierarchy);
		this._createTree(store);
	},
	
	_createProjectAreaSet: function(processAreaHierarchy) {
		var set = new Object();
		for (var i = 0; i < processAreaHierarchy.length; i++) {
			set[processAreaHierarchy[i].itemId] = true;
		}
		return set;
	},

	_createDataStore: function(processAreaHierarchy) {
		var store = new Object();
		store.label = "name";
		store.identifier = "itemId";
		store.items = this._filter(processAreaHierarchy);
		return store;
	},

	_filter: function(processAreaHierarchy) {
		var filter = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId;
		if (!filter)
			return processAreaHierarchy;
		var filtered = new Array();
		for (var i = 0; i < processAreaHierarchy.length; i++) {
			if (processAreaHierarchy[i].itemId == filter)
				filtered.push(processAreaHierarchy[i]);
		}
		return filtered;
	},
	
	_onFocus: function(node) {
		if (node && this._isValidSelection(node.item)) {
			this._currentSelection = node.item;
		} else {
			this._currentSelection = null;
		}
		this._updateEnablement();
	},
	
	_onType: function() {
		this._folder = this._trim(this._folderName.value);
		if (this._folder == "") {
			this._folder = null;
		}
		this._updateEnablement();
	},
	
	_updateEnablement: function() {
		if (this._currentSelection != null && this._folder != null) {
			this._okButton.disabled = false;
		} else {
			this._okButton.disabled = true;
		}
	},
	
	_isValidSelection: function(item) {
		return !this._projectAreaSet[item.itemId];
	},

	_isValidFolderName: function() {
		if (this._folder == null) {
			return false;
		}
		if (this._trim(this._folder) == "") {
			return false;
		}
		return true;
	},
	
	_onOk: function() {
		var selection = this._getSelectedItem();
		if (selection) {
			var args = {
        		name: this._trim(this._folder),
        		shared: this._shared,
        		teamAreaUUID: selection.itemId
        	};
        	if (this._parentFolder != null) {
        		args.parentFolder = this._parentFolder.folderUUID;
        	}
			var responseHandler = new ServiceResponseHandler(
					this,
					"_folderCreated",
					"_remoteError");
			ReportsClient.postSaveFolderDescriptor(responseHandler, args);
		}
	},
	
	_onCancel: function() {
		this.dialog.close();
	},
	
	_folderCreated: function() {
		this.dialog.close();
		this._controller.refreshNavigator();
	},
	
	_remoteError: function() {
		// TODO
		this.dialog.close();
	},

	_getSelectedItem: function() {
		return this._currentSelection;
	},
	
	// TODO: Move these to a utility class
	_trim: function(string) {
		return this._trimLeft(this._trimRight(string));
	},
	_trimLeft: function(string) {
		return string.replace(new RegExp("^[\\s]+", "g"), "");
	},
	_trimRight: function(string) {
		return string.replace(new RegExp("[\\s]+$", "g"), "");
	},
	
	_handleError: function(e) {
		alert(dojo.string.substitute(messages.error, [e]));
	}
});

})();