/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.app.view.navigation.QueryWidget");

dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.app.internal.common.ImagePool");

dojo.requireLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");

(function() { // module container

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ImagePool = com.ibm.team.reports.web.app.internal.common.ImagePool;

var DROPDOWN_HOVER = ImagePool.DD_ARROW_HOV;
var DROPDOWN = ImagePool.DD_ARROW;
var REPORT_OBJ = ImagePool.REPORT_OBJ;
var DEFAULT_REPORT_OBJ = ImagePool.DEFAULT_REPORT_OBJ;

/**
 * Widget to display a single report query
 */
dojo.declare("com.ibm.team.reports.web.app.view.navigation.QueryWidget", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/navigation/templates/QueryWidget.html"),
		//templateCssPath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/navigation/templates/QueryWidget.css"),

		RENAME_ICON_CLASS: "renameReportIcon",
		DELETE_ICON_CLASS: "deleteReportIcon",
		MAKE_DEFAULT_REPORT_ICON_CLASS: "makeDefaultReportIcon",
		
		constructor: function() {
			messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");
		},
		
		postCreate: function() {
			this._dropDownImage.src = DROPDOWN;
			this._query = this.query;
			this._controller = this.controller;
			this._parentWidget = this.parentWidget;
			this._canDeployQuery = this.canDeployQuery;
			this._defaultQuery = this.defaultQuery;
			if (this._defaultQuery != null && this._defaultQuery.queryUUID == this._query.queryUUID) {
				this._reportImage.src = DEFAULT_REPORT_OBJ;
			} else {
				this._reportImage.src = REPORT_OBJ;
			}
			this._setQuery(this._query);
		},

		/**
		 * Create the query contents.
		 */
		_setQuery: function(query) {
    		query.div = this.queryDiv;
            this._setQueryLink(this.queryDiv, query);
		},
		
		setDefaultQuery: function(defaultQuery) {
			this._defaultQuery = defaultQuery;
			if (this._defaultQuery != null && this._defaultQuery.queryUUID == this._query.queryUUID) {
				this._reportImage.src = DEFAULT_REPORT_OBJ;
			} else {
				this._reportImage.src = REPORT_OBJ;
			}
		},

		/**
		 * Adds a link for the query to the list.
		 */
		_setQueryLink: function (parentDiv, query) {
			// Construct the link here
			var anchor = document.createElement("a");
			anchor.className = "queryLink";
			anchor.href = "#action=com.ibm.team.reports.viewQuery&queryUUID=" + query.queryUUID;
			anchor.href += "&name=" + query.name;
			anchor.title = query.name;
			// Store the text node in a variable so we can change it if the 
			// folder gets renamed.
			this._textNode = document.createTextNode(query.name);
			anchor.appendChild(this._textNode);
			this._queryLinkTd.appendChild(anchor);
			
			// The selected query
			var selectedQuery = this._controller.getSelectedQuery();
			if (selectedQuery != null && selectedQuery == query.queryUUID) {
				this._treeItem.className = "treeItemSelected";
				this._isSelected = true;
			} else {
				this._treeItem.className = "treeItem";
				this._isSelected = false;
			}
			
			if (!dojo.isIE) {
				var itemRowDivTopLeft = document.createElement("span");
				itemRowDivTopLeft.className = "itemRowDivTopLeft";
				var itemRowDivTopRight = document.createElement("span");
				itemRowDivTopRight.className = "itemRowDivTopRight";
				this._treeItem.insertBefore(itemRowDivTopRight, this._treeItem.firstChild);
				this._treeItem.insertBefore(itemRowDivTopLeft, this._treeItem.firstChild);
			}

			// Hide the drop-down indicator initially
			this._dropDown.style.visibility = "hidden";

			// Hook the events for the drop-down indicator
			if (this._canDeployQuery) {
				this.connect(this._dropDownImage, "onmouseover", this._dropDownImageMouseOverHandler);
				this.connect(this._dropDownImage, "onmouseout", this._dropDownImageMouseOutHandler);
				this.connect(this._dropDown, "onclick", this._dropDownClickHandler);
			}
			
			if (!dojo.isIE) {
				var itemRowDivBottomLeft = document.createElement("span");
				itemRowDivBottomLeft.className = "itemRowDivBottomLeft";
				var itemRowDivBottomRight = document.createElement("span");
				itemRowDivBottomRight.className = "itemRowDivBottomRight";
				this._treeItem.appendChild(itemRowDivBottomLeft);
				this._treeItem.appendChild(itemRowDivBottomRight);
			}			
			
			// Blue hover support for the whole item
			this.connect(this._itemDiv, "onmouseover", this._itemDivMouseOverHandler);
			this.connect(this._itemDiv, "onmouseout", this._itemDivMouseOutHandler);
		},
		
		_dropDownImageMouseOverHandler: function(e) {
			e.preventDefault();
			this._dropDownImage.src = DROPDOWN_HOVER;
		},
		
		_dropDownImageMouseOutHandler: function(e) {
			e.preventDefault();
			this._dropDownImage.src = DROPDOWN;
		},
		
		_dropDownClickHandler: function(e) {
			e.preventDefault();
			this._openMenu(e);
		},
		
		_itemDivMouseOverHandler: function(e) {
			e.preventDefault();
			if (this._menu != null) {
				// We're already in the menu, ignore this
				this._mouseOutOccurred = false;
				return;
			}
			this._treeItem.className = "treeItemSelected";
			if (this._canDeployQuery) {
				this._dropDown.style.visibility = "visible";
			}
		},
		
		_itemDivMouseOutHandler: function(e) {
			e.preventDefault();
			if (this._menu != null) {
				// We're in the menu, don't remove the hover
				// but remember that we left
				this._mouseOutOccurred = true;
				return;
			}
			if (this._isSelected) {
				this._treeItem.className = "treeItemSelected";
			} else {
				this._treeItem.className = "treeItem";
			}
			if (this._canDeployQuery) {
				this._dropDown.style.visibility = "hidden";
			}
		},
		
		_openMenu: function(e) {
			if (this._menu) {
				this._closeMenu();
				return;
			}
			var menu = new dijit.Menu();
			menu.addChild(new dijit.MenuItem({label: messages.rename, iconClass: this.RENAME_ICON_CLASS, onClick: dojo.hitch(this,this._rename)}));
			menu.addChild(new dijit.MenuItem({label: messages.deleteMessage, iconClass: this.DELETE_ICON_CLASS, onClick: dojo.hitch(this,this._delete)}));
			menu.addChild(new dijit.MenuSeparator());
			menu.addChild(new dijit.MenuItem({label: messages.makeDefaultReport, iconClass: this.MAKE_DEFAULT_REPORT_ICON_CLASS, onClick: dojo.hitch(this,this._makeDefaultReport)}));
			var coords = dojo.coords(e.target, true);
			dijit.popup.open({ popup: menu, parent: this, x: coords.x + 10, y: coords.y + 10, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
			//register event handler that causes menu to close
			this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
			this._menu = menu;
			menu.focus();			
		},
		
		_closeMenu: function(e){
			if (e) {
				dojo.stopEvent(e);
			}
			if (this._mouseOutOccurred) {
				if (this._isSelected) {
					this._treeItem.className = "treeItemSelected";
				} else {
					this._treeItem.className = "treeItem";
				}
				this._dropDown.style.visibility = "hidden";
			}
			this.disconnect(this._menuCloseOnBody);
			dijit.popup.close(this._menu);
			this._menu = null;
		},
		
		_rename: function() {
			var newName = window.prompt(messages.newName, this._query.name);
			if (newName == null) {
				return;
			}
			newName = this._trim(newName);
			if (newName.length == 0) {
				alert(messages.invalidName);
				return;
			}
			this._newName = newName;
			var args = {
        		queryUUID: this._query.queryUUID,
        		name: newName
        	};
			var responseHandler = new ServiceResponseHandler(
					this,
					"_queryRenamed",
					"_remoteError");
			ReportsClient.postRenameQueryDescriptor(responseHandler, args);
		},
		
		_delete: function() {
			if (!confirm(messages.confirmDelete)) {
				return;
			}
			var args = {
        		queryUUID: this._query.queryUUID
        	};
			var responseHandler = new ServiceResponseHandler(
					this,
					"_queryDeleted",
					"_remoteError");
			ReportsClient.postDeleteQueryDescriptor(responseHandler, args);
		},
		
		_queryDeleted: function() {
			if (this._parentWidget != null) {
				this._parentWidget.removeChild(this);
			}
		},
		
		_queryRenamed: function() {
			if (this._newName != null) {
				this._textNode.nodeValue = this._newName;
			}
			this._newName = null;
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._itemDiv, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			alert(dojo.string.substitute(messages.errorWithMessage, [error.message]));
		},
		
		_makeDefaultReport: function() {
			var args = {
				queryUUID: this._query.queryUUID
			};
			var responseHandler = new ServiceResponseHandler(
					this,
					"_querySetAsDefault",
					"_remoteError");
			ReportsClient.postSetDefaultQueryDescriptor(responseHandler, args);
		},
		
		_querySetAsDefault: function() {
			alert(dojo.string.substitute(messages.newDefaultReport, [this._query.name]));
			this._controller.setDefaultQuery(this._query);
		},
		
		_trim: function(string) {
			return this._trimLeft(this._trimRight(string));
		},
		_trimLeft: function(string) {
			return string.replace(new RegExp("^[\\s]+", "g"), "");
		},
		_trimRight: function(string) {
			return string.replace(new RegExp("[\\s]+$", "g"), "");
		}
	}
);

})();