/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.app.view.plan.TabCharts");

dojo.require("dojo.i18n");

dojo.require("dijit._Templated");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.apt.web.ui.contribution.ContributedTab");

dojo.requireLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");

(function() {
	
var ReportsClient	= com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

dojo.declare("com.ibm.team.reports.web.app.view.plan.TabCharts",
			 [com.ibm.team.apt.web.ui.contribution.ContributedTab, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_renderedReportDiv: null,

	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");
		this._notYetShown= true;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------	

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/plan/templates/TabCharts.html"),

	// ---- ContributedTab.js implementation -------------------------------------------------------------------------------

	onShowTab: function() {
		if (this._notYetShown) {
			var responseHandler = new ServiceResponseHandler(
				this,
				"_existenceCallback",
				"_remoteError");		
			
			var clientArgs = {
				reportId: "apt.WorkItems"
			};
			ReportsClient.getDoesReportDescriptorExist(responseHandler, clientArgs);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_existenceCallback: function(result) {
		var intervalName = this.getPlanIteration().label;
		var teamAreaPath = this.getPlanTeamArea().path;
		
		var clientArgs;
		if (result == "true") {
			var clientArgs= {
				reportUUID: "apt.WorkItems",
				webui: true,
				parameterName: ["Interval", "TeamAreaName"],
				parameterValue: ["'" + intervalName + "'", "'" + teamAreaPath + "'"]
			};
		} else {
			var clientArgs = {
				reportUUID: "workitems.OpenWorkItemsByType",
				webui: true,
				parameterName: ["Interval", "TeamAreaName"],
				parameterValue: ["'" + intervalName + "'", "'" + teamAreaPath + "'"]
			};
		}
		var responseHandler = new ServiceResponseHandler(
			this,
			"_displayRenderedReport",
			"_remoteError");		
	
		ReportsClient.postRenderReport(responseHandler, clientArgs);
	},
	
	_displayRenderedReport: function(result) {
		if (navigator.appVersion.indexOf("MSIE")!=-1) {
			// Insert innerHTML directly. That, coupled with defer tags
			// on the scripts, causes new Javascript to be executed properly.
			this._renderedReportDiv.innerHTML = result;
		} else {
			// Create a new node and appendChild so javascript gets executed.
			var newNode = document.createElement('div');
			newNode.innerHTML = result;
			dojox.data.dom.removeChildren(this._renderedReportDiv);
			this._renderedReportDiv.appendChild(newNode);
		} 
	},
	
	_remoteError: function(error) {
		alert(dojo.string.substitute(messages.errorRetrievingChart, [error.message]));
	}
	
});

})();

