/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.app.view.report.ReportViewer");

dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.requireLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");

// Global variable to store the controller. Required to make
// the controller available to the code generated by the REST
// service.
reportController = null;

(function() { // module container

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var session = com.ibm.team.repository.web.client.session;

/**
 * Widget to display a report.
 */
dojo.declare("com.ibm.team.reports.web.app.view.report.ReportViewer", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/report/templates/ReportViewer.html"),
		//templateCssPath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/report/templates/ReportViewer.css"),
		
		constructor: function() {
			messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");
		},
		
		// Arguments:
		//     queryUUID - mandatory UUID of the query
		postCreate: function() {
			reportController = this.controller;
			
			// Handle non-query reports
			if (this.isReport == true) {
				if (this.parameterName == null || this.parameterName == undefined) {
					this.parameterName = [];
				}
				if (this.parameterValue == null || this.parameterValue == undefined) {
					this.parameterValue = [];
				}
				var renderedReportArgs = {
					reportUUID: this.reportUUID,
					parameterName: this.parameterName,
					parameterValue: this.parameterValue,
					showArchived: reportController.getIncludeArchived(),
					webui: true
				};
				
				this._removeChildren(this.parametersDiv);
				this._removeChildren(this.renderedReportDiv);
				this._removeChildren(this.loadingDiv);
				this.controller.startOperation(messages.loadingReport);
				this._gotParametersResponse = false;
				this._gotRenderedResponse = false;
				var responseHandler = new ServiceResponseHandler(
						this,
						"_displayParametersSection",
						"_remoteError");
				ReportsClient.getReportParametersSection(responseHandler, renderedReportArgs);
				responseHandler = new ServiceResponseHandler(
						this,
						"_displayRenderedQuery",
						"_remoteError");
				ReportsClient.postRenderReport(responseHandler, renderedReportArgs);
				return;
			}
			
			// Now handle regular query
			var renderedQueryArgs = {
				queryUUID : this.queryUUID,
				showArchived: reportController.getIncludeArchived(),
				webui: true
			};
			
			this._removeChildren(this.parametersDiv);
			this._removeChildren(this.renderedReportDiv);
			this._removeChildren(this.loadingDiv);
			this.controller.startOperation(messages.loadingReport);
			this._gotParametersResponse = false;
			this._gotRenderedResponse = false;
			var responseHandler = new ServiceResponseHandler(
					this,
					"_displayParametersSection",
					"_remoteError");
			ReportsClient.getQueryParametersSection(responseHandler, renderedQueryArgs);
			responseHandler = new ServiceResponseHandler(
					this,
					"_displayRenderedQuery",
					"_remoteError");
			ReportsClient.getRenderedQuery(responseHandler, renderedQueryArgs);
		},

		_displayRenderedQuery: function(result) {
			// Check if the page is still displaying
			if (!dojo.isDescendant(this.renderedReportDiv, document.body)) {
				// Not still displaying, ignore the callback
				return;
			}
			this._renderedQuery = null;
			if (!this._gotParametersResponse) {
				// Store it for later
				this._renderedQuery = result;
				this._gotRenderedResponse = true;
				return;
			}
			// Now have both responses, remove the loading spinner
			this.controller.finishOperation(messages.loadingReport);
			this._removeChildren(this.loadingDiv);

			if (navigator.appVersion.indexOf("MSIE")!=-1) {
				// Insert innerHTML directly. That, coupled with defer tags
				// on the scripts, causes new Javascript to be executed properly.
				this.renderedReportDiv.innerHTML = result;
			} else {
				// Create a new node and appendChild so javascript gets executed.
				var newNode = document.createElement('div');
				newNode.innerHTML = result;
				this._removeChildren(this.renderedReportDiv);
				this.renderedReportDiv.appendChild(newNode);
			}
		},

		_displayParametersSection: function(result) {
			// Check if the page is still displaying
			if (!dojo.isDescendant(this.parametersDiv, document.body)) {
				// Not still displaying, ignore the callback
				return;
			}
			this._gotParametersResponse = true;
			if (this._gotRenderedResponse) {
				// Now have both responses, remove the loading spinner
				this.controller.finishOperation(messages.loadingReport);
				this._removeChildren(this.loadingDiv);
			} else {
				this.controller.finishOperation(messages.loadingReport);
				this.loadingDiv.appendChild(document.createTextNode(messages.loading));
			}
			if (navigator.appVersion.indexOf("MSIE")!=-1) {
				// Insert innerHTML directly. That, coupled with defer tags
				// on the scripts, causes new Javascript to be executed properly.
				this.parametersDiv.innerHTML = result;
			} else {
				// Create a new node and appendChild so javascript gets executed.
				var newNode = document.createElement('div');
				newNode.innerHTML = result;
				this._removeChildren(this.parametersDiv);
				this.parametersDiv.appendChild(newNode);
			} 
			if (this._gotRenderedResponse && (this._renderedQuery != null)) {
				if (navigator.appVersion.indexOf("MSIE")!=-1) {
					// Insert innerHTML directly. That, coupled with defer tags
					// on the scripts, causes new Javascript to be executed properly.
					this.renderedReportDiv.innerHTML = this._renderedQuery;
				} else {
					// Create a new node and appendChild so javascript gets executed.
					var newRenderedNode = document.createElement('div');
					newRenderedNode.innerHTML = this._renderedQuery;
					this._removeChildren(this.renderedReportDiv);
					this.renderedReportDiv.appendChild(newRenderedNode);
				} 
			}
		},
		
		_remoteError: function(error) {
			this.controller.finishOperation("Loading Report");
			this._removeChildren(this.loadingDiv);
			this.loadingDiv.appendChild(document.createTextNode(dojo.string.substitute(messages.anErrorOccurred, [error.message])));
		},
		
		_removeChildren: function(element) {
			while (element.firstChild) {
			    element.removeChild(element.firstChild);
			}
		}
	}
);

})();