/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.ViewletPageHelper");

dojo.require("com.ibm.team.dashboard.web.client.internal.ColumnHelper");
dojo.require("com.ibm.team.dashboard.web.client.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletHelper");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

(function() {

/*
 * Aliases
 */
var ColumnHelper = com.ibm.team.dashboard.web.client.internal.ColumnHelper;
var Constants = com.ibm.team.dashboard.web.client.internal.Constants;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ViewletHelper = com.ibm.team.dashboard.web.client.internal.ViewletHelper;

/*
 * Models a viewlet page.
 */
dojo.declare("com.ibm.team.dashboard.web.client.internal.ViewletPageHelper",
	null, {
	
	/*
	 * Constructs a viewlet page.
	 */
	constructor: function(/* ViewletPage */ rawHelper, /* DashboardItem */ item) {
		this._item = item;
		this._id = rawHelper.id;
		this._title = rawHelper.title;
		this._columns = new Array();
		if (rawHelper.columns) {
			for (var i=0;i<rawHelper.columns.length;i++) {
				this._columns.push(new ColumnHelper(rawHelper.columns[i], this._item));
			}
		}
	},
	
	/*
	 * Adds a viewlet to the viewlet page.
	 */
	addViewlet: function(/* ViewletHelper */ viewletHelper, /* number */ destColumnIndex, /* number */ destViewletIndex) {
		this._columns[destColumnIndex].getViewlets().splice(destViewletIndex, 0, viewletHelper);
		
		// save state
		var modification = {
			type: Constants.MODIFICATION_TYPE_ADD_VIEWLET,
			viewletPageId: this.getId(),
			viewletId: viewletHelper.getId(),
			title: viewletHelper.getTitle(),
			definitionId: viewletHelper.getDefinitionId(),
			icon: viewletHelper.getIcon(),
			userTitle: viewletHelper.isUserTitle(),
			preferences: viewletHelper.getPreferences(),
			showBackground: viewletHelper.isShowBackground(),
			destColumnIndex: destColumnIndex,
			destViewletIndex: destViewletIndex
		};
		var memento = viewletHelper.getMemento();
		if (memento) {
			var jsonMemento = memento.serialize();
			if (jsonMemento) {
				modification.memento = jsonMemento;
			}
		}
		var trim = viewletHelper.getTrim();
		if (trim || trim == 0) {
			modification.trim = trim;
		}
		var scope = viewletHelper.getScope();
		if (scope || scope == 0) {
			modification.scope = scope;
		}
		var scopeItem = viewletHelper.getScopeItem();
		if (scopeItem) {
			modification.scopeItem = scopeItem.itemId;
		}
		this._item.addModification(modification);
	},

	/*
	 * Returns the dashboard item that contains this viewlet page.
	 */
	/* DashboardItem */ getDashboardItem: function() {
		return this._item;
	},
	
	/*
	 * Returns the viewlet id.
	 */
	/* string */ getId: function() {
		return this._id;
	},

	/*
	 * Returns the viewlet's title.
	 */
	/* string */ getTitle: function() {
		return this._title;
	},

	/*
	 * Sets the viewlet's title.
	 */
	setTitle: function(/* string */ title) {
		this._title = title;
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_PAGE_TITLE, viewletPageId:this.getId(), title:title});
	},
	
	/*
	 * Returns an array of columns in the page.
	 */
	/* ColumnHelper[] */ getColumns: function() {
		return this._columns;
	}
});

})();