/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.BuildsViewlet");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.RelativeDateFormat");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedViewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "BuildsMessages");

(function() {

/*
 * Aliases
 */
var RelativeDateFormat = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.RelativeDateFormat;

var JazzFeedViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedViewlet;
var FeedClient = com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

var messages;

/*
 * A build specific flavour of the JazzFeedViewlet.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.BuildsViewlet",
	JazzFeedViewlet, {

	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "BuildsMessages");
		this.setTrackReadMarks(false);
	},

	getFilters: function() {
		return ["build"];
	},
	
	getNoItemsMessage: function() {
		return messages.noBuilds;
	},
	
	getShowActiveBuilds: function() {
		return this.getPreference("showActive");
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var scopeChanged = false;
		var updateTitle = false;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			scopeChanged = true;
		}
		if (updateTitle || newSettings.userTitle != oldSettings.userTitle)
			this._updateTitle();
		if (scopeChanged || (newSettings.scope != DashboardConstants.SCOPE_CONTRIBUTOR && newSettings.scopeSubteams != oldSettings.scopeSubteams)
			|| oldSettings.preferences.showActive != newSettings.preferences.showActive) {
			this.reset(true);
			this.getSite().refresh();
		}
		else {
			this.update();
			this.updateTotalPageFooter();
		}
	},
	
	_updateTitle: function() {
		this.getSite().setTitle(this.getScopeItem() ? dojo.string.substitute(messages.scopedTitle, [this.getScopeItem().name]) : messages.title);
	},
	
	fetchTooltip: function (entry, srh) {
		var item = entry.items[0];
		var url = item.linkURL;
		var itemId = url.substring(url.lastIndexOf("/"));
		url =  URLUtility.getContextRoot() + "/resource/virtual/build/result" + itemId + "/contributions?type=com.ibm.team.build.common.model.IBuildResultContribution.artifact&type=com.ibm.team.build.common.model.IBuildResultContribution.log";
		var bindArgs = {
			url: url,
			handleAs: "json",
			headers: {accept: "text/json"},
			handle: function(response, ioArgs){
				if (response instanceof Error) {
					item.description = " ";
				}
				else {
					var date = new Date(response.buildResult.buildStartTime);
					var tipHtml = "<div class=\"com-ibm-team-dashboard-BuildsViewletTooltip\">";
					tipHtml += "<div class=\"section\"><span class=\"heading\">" + messages.startTime + "</span>: " + RelativeDateFormat.formatDif(date, true, true) + "</div>";
					tipHtml += "<div class=\"section\"><span class=\"heading\">" + messages.duration + "</span>: " + RelativeDateFormat.formatTime(response.buildResult.buildTimeTaken) + "</div>";
					var logHtml = "<div class=\"section\"><span class=\"heading\">" + messages.logs + "</span>:";
					var downloadHtml = "<div class=\"section\"><span class=\"heading\">" + messages.downloads + "</span>:";
					if (response.contributions) {
						for (var i = 0; i < response.contributions.length; i++) {
							delete filename;
							delete uri;
							if (response.contributions[i].contentUri)
								var uri = "../../" + response.contributions[i].contentUri;
							else if (response.contributions[i].contribution.extendedContributionProperties) {
								for (var j = 0; j < response.contributions[i].contribution.extendedContributionProperties.length; j++) {
									if (response.contributions[i].contribution.extendedContributionProperties[j].name == "com.ibm.team.build.common.model.IBuildResultContribution.url")
										var uri = response.contributions[i].contribution.extendedContributionProperties[j].value;
								}
							}
							if (!uri)
								continue;
							if (response.contributions[i].contribution.extendedContributionTypeId == "com.ibm.team.build.common.model.IBuildResultContribution.log") {
								var statusIcon = response.contributions[i].contribution.contributionStatus == "ERROR" ? "errorLog" : "log";
								logHtml = logHtml + "<a href=\"" + uri + "\" class=\"icon " + statusIcon + "\">" + response.contributions[i].contribution.label + "</a>";
								var hasLogs = true;
							} else if (response.contributions[i].contribution.extendedContributionTypeId == "com.ibm.team.build.common.model.IBuildResultContribution.artifact") {
								downloadHtml = downloadHtml + "<a href=\"" + uri + "\" class=\"icon artifact\">" + response.contributions[i].contribution.label + "</a>";
								var hasDownloads = true;
							}
						}
					}
					tipHtml = tipHtml + (hasDownloads ? downloadHtml + "</div>" : "") + (hasLogs ? logHtml + "</div>" : "") + "</div>";
					item.description = tipHtml;
				}
				srh.doCallback({entries: [{
					items: [item]
				}]});
			}
		};
		var xhr = dojo.xhrGet(bindArgs).ioArgs.xhr;
		this.getServiceTracker().register(xhr, srh);
	}
});

})();
