/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.DescriptionViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.DescriptionClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "DescriptionMessages");

(function() {

var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var DescriptionClient = com.ibm.team.dashboard.viewlets.web.client.internal.DescriptionClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.DescriptionViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/DescriptionViewlet.html"),

	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "DescriptionMessages");
		this.loading = messages.loading;
	},

	init: function() {
		this._updateTitle();
	},
	
	refresh: function() {
		// handle Auto(none) scope when no team areas
		if (!this.getScopeItem()) {
			// clear contents and show msg
			dojo.addClass(this._summary, "hidden");
			this._details.innerHTML = "";
			this._details.className = "";
			if (this._summary.nextSibling != this._details)
				this._summary.parentNode.removeChild(this._summary.nextSibling);
			this._details.appendChild(this._createNoScopeMsg());
			this.getSite().doneRefresh();
			return;
		}
		var self = this;
		this.getSite().setLoading(true);
		var serviceObj = {
			success: function(descriptions) {
				self._descriptions = descriptions;
				self.update();
				self.getSite().setLoading(false);
				self.getSite().doneRefresh();
			},
			failure: function(errorObj) {
				self.getSite().doneRefresh();
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [errorObj]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DescriptionClient.getDescriptionsForProcessArea(this.getScope(), this.getScopeItem().itemId, srh, this.getServiceTracker());
	},
	
	update: function() {
		this._summary.innerHTML = "";
		this._summary.className = "summary";
		this._details.innerHTML = "";
		this._details.className = "";
		if (this._summary.nextSibling != this._details)
			this._summary.parentNode.removeChild(this._summary.nextSibling);
		dojo.removeClass(this._summary, "padded");
		if (this._descriptions && (this._descriptions[0] || this._descriptions[1])) {
			if (this._descriptions[0]) {
				this._summary.appendChild(document.createTextNode(this._descriptions[0]));
				if (this._descriptions[1]) 
					this._summary.parentNode.insertBefore(document.createElement("br"), this._details);
			}
			if (this._descriptions[1]) 
				this._populateDetails(this._descriptions[1]);
		}
		else {
			if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA)
				this._details.appendChild(document.createTextNode(messages.noDescTeam));
			else
				this._details.appendChild(document.createTextNode(messages.noDescProject));
		}
		if (!this._summary.firstChild)
			dojo.addClass(this._summary, "hidden");
		if (!this._details.firstChild)
			dojo.addClass(this._details, "hidden");
	},
	
	_populateDetails: function(/* string */ input) {
		input = dojo.trim(input.replace("\r",""));
		for (var i = input.indexOf("\n"); i != -1; i = input.indexOf("\n")) {
			this._details.appendChild(document.createTextNode(input.substring(0,i)));
			this._details.appendChild(document.createElement("br"));
			input = input.substring(i+1);
		}
		this._details.appendChild(document.createTextNode(input));
	},
	
	settingsChanged: function(/* object */ oldSettings, /* object */ newSettings) {
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			this._updateTitle();
			this.getSite().refresh();
		}
		else 
			this.update();
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var nodes = I18nUtil.addMessage(editSpan, messages.notScopedEdit);
			this.connect(nodes[0], "onclick", this._handleToggleSettings);
			msgSpan.appendChild(editSpan);
		}
		return msgSpan;
	},
	
	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},

	_updateTitle: function() {
		this.getSite().setTitle(this.getScopeItem() ?  dojo.string.substitute(messages.titleScoped, [this.getScopeItem().name]) : messages.title);
	}
	
});

})();