/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet");

/*
 * Imports 
 */
dojo.require("dojo.i18n");  
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "TrendReportMessages");

(function() {

/*
 * Aliases 
 */
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

/*
 * TODOs
 */
// Create setter: getTitleHyperlinkURL

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet",
	Viewlet, {

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/TrendReportViewlet.html"),
	
	/*
	 * Static Variable Container 
	 */
	_staticCommon: function() {}, 

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return TrendReportViewlet
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "TrendReportMessages");
		
	},
	
	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	init: function() {

		// Static Variables //------------------------------------------------->

		this._initializeStaticCommons();

		// Instance Variables //----------------------------------------------->
		
		this._viewletID = this._staticCommon._instanceCount;
		
		this._reportCache = new Array();
		this._reportKeyStack = new Array();
		
		this._clickThroughURL = "#action=jazz.viewPage&id=com.ibm.team.reports";

		this._reportUUID = null;

		this._projectAreaName = null;

		this._messageNoResults = this.messages.msgNoResultsFound;

		this._isExpressEdition = this._checkIfIsExpressEdition();

		// Setup //------------------------------------------------------------>

		// Set the default report for querying parameters
		this._setDefaultReport();

	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return boolean
 	 */	
	_checkIfIsExpressEdition: function() {
		if (this.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA)) {
			// Team dashboards are available
			// Standard edition
			return false;
		}
		// Team dashboards are NOT available
		// Express edition
		return true;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	update: function() {
		// Called to update UI (e.g. Zoom)
		this.getSite().refresh();
	},

	/*
 	 * Public Method
 	 * 
 	 * Recieves notification when the setting areas is opened or closed.
 	 * 
 	 * @param boolean opened
 	 * @return void
 	 * @overrides super
 	 */		
	settingsAreaToggled: function(opened) {
		var closed = (opened == false);
		// Check if the settings area was closed
		if (closed) {
			// Reset the cascading parameters to force a GET each time
			this._resetCascadingParameters();
			return;
		}
		// Settings area is open
		// Get the report parameters
		this._getParameters();
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getTitleHyperlinkURL: function() {
		if (this._isExpressEdition) {
			// No Reports Web UI vertical in Express editions
			// Disable title hyperlink
			return null;
		}
		return this._clickThroughURL;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return boolean
 	 * @overrides super
 	 */		
	hasZoomableContent: function() {
		if (this.getSite().isLoading()) {
			return false;
		}
		return true; 
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * @return void
 	 * @overrides super
 	 */		
	settingsChanged: function(oldSettings, newSettings) {
		// If the new settings is different from the old settings then 
		// refresh the viewlet content
		if (this._isSettingsEqual(oldSettings, newSettings) == false) {
			this.getSite().refresh();
		}
	},


	// Private Methods -------------------------------------------------------->
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticCommons: function() {

		// Track the number of viewlet instances
		if (this._staticCommon._instanceCount == undefined) {
			this._staticCommon._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._staticCommon._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._staticCommon._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet";

		this._staticCommon._REPORT_CACHE_SIZE = 20;

		this._staticCommon._PREFERENCE_ID_REPORT = "report";		

		this._staticCommon._PREFERENCE_OPTION_VALUE_ALL = "<<All>>";

		this._staticCommon._SPECIAL_PROJECT_AREA = "{Current Project Area}";
		this._staticCommon._SPECIAL_CURRENT_ITERATION = "{Current Iteration}";
		this._staticCommon._SPECIAL_MY_TEAM_AREAS = "{My Team Areas}";

		this._staticCommon._REPORT_DOM_ID = "trend_report";
		this._staticCommon._CACHE_STATUS_DOM_ID = "cache_status";

		this._staticCommon._REPORT_REST_SERVICE_URI = "com.ibm.team.reports.common.internal.service.IReportRestService";

		// Work Item Micro Reports
		this._staticCommon._WORK_ITEMS_OPEN_VS_CLOSED = "workitems.MicroOpenVsClosedWorkItems";
		this._staticCommon._WORK_ITEMS_OPEN_BY_TYPE = "workitems.MicroOpenWorkItemsByType";
		this._staticCommon._WORK_ITEMS_OPEN_BY_PRIORITY = "workitems.MicroOpenWorkItemsByPriority";
		this._staticCommon._WORK_ITEMS_BLOCKING = "workitems.MicroBlockingWorkItems";
		this._staticCommon._WORK_ITEMS_NEW_BY_SEVERITY = "workitems.MicroNewWorkItemsBySeverity";
		this._staticCommon._WORK_ITEMS_CLOSED_BY_PRIORITY = "workitems.MicroClosedWorkItemsByPriority";
		this._staticCommon._WORK_ITEMS_BURNDOWN = "workitems.MicroBurndown";
		this._staticCommon._WORK_ITEMS_STORY_POINTS = "workitems.MicroStoryPoints";
		this._staticCommon._WORK_ITEMS_STORY_BY_ITERATION = "workitems.MicroStoryPointsByIteration";
		this._staticCommon._WORK_ITEMS_TEAM_VELOCITY = "workitems.MicroTeamVelocity";
		this._staticCommon._WORK_ITEMS_CLOSED_DAILY = "workitems.MicroClosedWorkItemsDaily";
		this._staticCommon._WORK_ITEMS_COMPARISON = "workitems.MicroWorkItemComparison";
		this._staticCommon._WORK_ITEMS_DEFERRED = "workitems.MicroDeferredWorkItems";

		// Build Micro Reports
		this._staticCommon._REPORT_BUILD_HEALTH = "build.MicroBuildHealth";
		this._staticCommon._REPORT_BUILD_DURATION = "build.MicroBuildDuration";
		this._staticCommon._REPORT_JUNIT_TESTS = "build.MicroJUnitTests";
		this._staticCommon._REPORT_CODE_COVERAGE = "build.MicroCodeCoverage";
		this._staticCommon._REPORT_JUNIT_FAILURES = "build.MicroJUnitFailures";
		this._staticCommon._REPORT_COMPILATION_WARNINGS = "build.MicroCompilationWarnings";
		this._staticCommon._REPORT_STATIC_ANALYSIS = "build.MicroStaticAnalysis";
		this._staticCommon._REPORT_FREQUENT_TEST_FAILURES = "build.MicroFrequentTestFailures";
		this._staticCommon._REPORT_FREQUENT_TEST_REGRESSIONS = "build.MicroFrequentTestRegressions";

		// Mapping of micro reports to normal reports
		this._staticCommon._microReportIDMap = new Array();
		// Work Items
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_OPEN_VS_CLOSED] = "workitems.OpenVsClosedWorkItems";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_OPEN_BY_TYPE] = "workitems.OpenWorkItemsByType";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_OPEN_BY_PRIORITY] = "workitems.WorkItemsByPriority";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_BLOCKING] = "workitems.BlockingWorkItems";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_NEW_BY_SEVERITY] = "workitems.NewWorkItemsBySeverity";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_CLOSED_BY_PRIORITY] = "workitems.WorkItemsByPriority";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_BURNDOWN] = "workitems.Burndown";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_STORY_POINTS] = "workitems.StoryPoints";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_STORY_BY_ITERATION] = "workitems.StoryPointsByIteration";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_TEAM_VELOCITY] = "workitems.TeamVelocity";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_CLOSED_DAILY] = "workitems.ClosedWorkItemsDaily";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_COMPARISON] = "workitems.WorkItemComparison";
		this._staticCommon._microReportIDMap[this._staticCommon._WORK_ITEMS_DEFERRED] = "workitems.DeferredWorkItems";
		
		// Builds
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_BUILD_HEALTH] = "build.BuildHealth";
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_BUILD_DURATION] = "build.BuildHealth";
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_JUNIT_TESTS] = "build.CodeCoverage";
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_CODE_COVERAGE] = "build.CodeCoverage";
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_JUNIT_FAILURES] = "build.CodeHealth";
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_COMPILATION_WARNINGS] = "build.CodeHealth";
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_STATIC_ANALYSIS] = "build.CodeHealth";
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_FREQUENT_TEST_FAILURES] = "build.FrequentTestFailures";
		this._staticCommon._microReportIDMap[this._staticCommon._REPORT_FREQUENT_TEST_REGRESSIONS] = "build.FrequentTestRegressions";

		this._staticCommon._URI_REPORT_ACTION = "#action=com.ibm.team.reports.viewReport&reportUUID=";
		this._staticCommon._URI_REPORT_PARAMETER_NAME = "&parameterName=";
		this._staticCommon._URI_REPORT_PARAMETER_VALUE = "&parameterValue=";

		this._staticCommon._DTO_PARAMETERS_FAILURE = "error";

		// Multi-select parameter separator
		this._staticCommon._MULTI_PARAMETER_SEPARATOR = ",";

		// Common zoom parameter name used for REST service calls and drill-in URL operations
		this._staticCommon._PARAMETER_NAME_ZOOM = "Zoom";
		
		// Dashboard drill-in URL components
		this._staticCommon._DASHBOARD_DRILL_ACTION_VIEW = "#action=com.ibm.team.dashboard.viewDashboard";
		this._staticCommon._DASHBOARD_DRILL_ACTION_VIEW_ALL = "#action=com.ibm.team.dashboard.viewAllDashboards";
		this._staticCommon._DASHBOARD_DRILL_IN_PARAM_TEAM = "&team=";
		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_resetCascadingParameters: function() {
		// Subclasses to override
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_getParameters: function() {
		// Subclasses to override
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * @return boolean
 	 */		
	_isSettingsEqual: function(oldSettings, newSettings) {
		// Subclasses to override
		return true;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @return void
 	 */	
	_addParametersToNode: function(node) {
		// Subclasses to override
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String message
 	 * 
 	 * @return void
 	 */	
	_setMessageNoResults: function(message) {
		this._messageNoResults = message;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_setDefaultReport: function() {
		// Get the current report preference value
		this._reportUUID = this.getPreference(this._staticCommon._PREFERENCE_ID_REPORT);
		// Verify that the preference is defined
		try {
			this._isStringDefined(this._reportUUID, true, "_reportUUID");
		} catch(errorObject) {
			this._logError(errorObject);
			// BUG: Error message gets overidden in viewlet content area
			this._displayErrorMessage(this.messages.msgUnknownReportType, errorObject);
		}		
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array
 	 */		
	_getPreferenceFailureList: function() {
		var result = new Object();
		// Create the values
		result.options = new Array();
		// Add Select option
		this._addSelectOption(result.options);
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * Union operation.
 	 * Merge options from the second array into the first array
 	 * 
 	 * @param Object preferenceObj1
	 *				. Array options : Object
	 *					. String label
	 * 					. String value
 	 * @param Object preferenceObj2
	 *				. Array options : Object
	 *					. String label
	 * 					. String value
  	 * @return Object 
	 *				. Array options : Object
	 *					. String label
	 * 					. String value
 	 */		
	_mergePreferences: function(preferenceObj1, preferenceObj2) {
		for (var index = 0; index < preferenceObj2.options.length; index++) {
			var option = preferenceObj2.options[index];
			preferenceObj1.options.push(option);
		}
		return preferenceObj1;
	},

	/*
 	 * Private Method
 	 * 
 	 * Intersection operation.
 	 * Merge options from the second array into the first array
 	 * 
  	 * @param Array valueMap : [String]=boolean
 	 * @param Object preferenceObj1
	 *				. Array options : Object
	 *					. String label
	 * 					. String value
 	 * @param Object preferenceObj2
	 *				. Array options : Object
	 *					. String label
	 * 					. String value
  	 * @return Object 
	 *				. Array options : Object
	 *					. String label
	 * 					. String value
 	 */		
	_integratePreferences: function(valueMap, preferenceObj1, preferenceObj2) {
		// Add the second set of preferences to the first set of preferences
		for (var index = 0; index < preferenceObj2.options.length; index++) {
			var option = preferenceObj2.options[index];
			var value = option.value;
			// Do not allow duplicate entries
			if ((value in valueMap) == false) {
				preferenceObj1.options.push(option);
				valueMap[value] = true;
			}
		}

		return preferenceObj1;
	},

	/*
 	 * Private Method
 	 * 
 	 * 
 	 * @param Object preferenceObj
	 *				. Array options : Object
	 *					. String label
	 * 					. String value
  	 * @return Array : [String]=boolean
 	 */		
	_createPreferenceValueMap: function(preferenceObj) {
		var valueMap = new Array();
		for (var index = 0; index < preferenceObj.options.length; index++) {
			var value = preferenceObj.options[index].value;
			valueMap[value] = true;
		}
		
		return valueMap;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object parameterObject
	 *			. Array values : String
	 *			. [Array labels : String]
	 * @param String normalizeLabelMethodName 
	 * 					Function (String)
	 * 
  	 * @return Object
 	 */		
	_adaptParametersToPreferences: function(parameterObject, normalizeLabelMethodName) {
		// Preference container
		var result = this._createPreferenceContainer();
		
		// Validate object
		if (this._isValidParamaterObject(parameterObject) === false) {
			return result;
		}
		// Determine whether to process labels
		var hasLabels = this._hasLabelsParameterObject(parameterObject);
		var labels = null;
		
		// Normalize labels
		if (this._isStringDefined(normalizeLabelMethodName, false, "normalizeLabelMethodName")) {
			if (hasLabels) {
				labels = this._normalizeLabels(normalizeLabelMethodName, parameterObject.labels);
			} else {
				labels = this._normalizeLabels(normalizeLabelMethodName, parameterObject.values);
			}
		} else {
			if (hasLabels) {
				labels = parameterObject.labels;
			} else {
				labels = parameterObject.values;
			}
		}
		
		// Convert parameters to preferences
		for (var i = 0; i < parameterObject.values.length; i++) {
			// Populate preference
			result.options[i] = this._createPreferenceEntry(
				labels[i], 
				parameterObject.values[i]);
		}

		return result;
	},

	/*
 	 * Private Method
 	 * 
	 * @param String normalizeLabelMethodName 
	 * 					Function (String)
 	 * @param Array values : String
	 * 
  	 * @return Array : String
 	 */		
	_normalizeLabels: function(normalizeLabelMethodName, values) {
		var labels = new Array();
		// Convert parameters to preferences
		for (var index = 0; index < values.length; index++) {
			labels[index] = this[normalizeLabelMethodName](values[index]);
		}
		return labels;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object parameterObject
	 *			. Array values : String
	 *			. [Array labels : String]
 	 * @return boolean
 	 */		
	_isValidParamaterObject: function(parameterObject) {
		// Validate object
		if (this._isObjectDefined(parameterObject, false, "parameterObject") === false) {
			return false;
		}
		// Validate values
		if (this._hasProperty(parameterObject, "values", false, "parameterObject") === false) {
			return false;
		} else if (this._isArray(parameterObject.values, false, "values") === false) {
			return false;
		}		
		return true;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object parameterObject
	 *			. Array values : String
	 *			. [Array labels : String]
 	 * @return boolean
 	 */		
	_hasLabelsParameterObject: function(parameterObject) {
		if (this._hasProperty(parameterObject, "labels", false, "parameterObject") === false) {
			return false;
		} else if (this._isArray(parameterObject.labels, false, "labels") === false) {
			return false;
		}
		return true;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String label
 	 * @param String value
 	 * @return Object
 	 * 				. String label
 	 * 				. String value
 	 */		
	_createPreferenceEntry: function(label, value) {
		var entry = new Object();
		entry.label = label;
		entry.value = value;
		
		return entry;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Object
 	 * 				. Array options : [Number]=Object
 	 * 					. String label
 	 * 					. String value
 	 */		
	_createPreferenceContainer: function() {
		var result = new Object();
		result.options = new Array();
		
		return result;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Object parameterObject
	 *			. Array values : String
	 *			. [Array labels : String]
 	 * @param Number index	 
 	 * @return String
 	 */		
	_getParameterValue: function(parameterObject, index) {
		return parameterObject.values[index];
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Array options
 	 * @return void
 	 */		
	_addSelectOption: function(options) {
		// Add Select option
		var entry = new Object();
		entry.label = this.messages.msgSelect;
		entry.value = "";
		options.unshift(entry);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Array options
 	 * @return void
 	 */		
	_addNoValuesOption: function(options) {
		var entry = new Object();
		entry.label = this.messages.msgNoAvailableOptions;
		entry.value = "";
		options.unshift(entry);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Array options
 	 * @return void
 	 */		
 	 // DEAD CODE
	_addAllOption: function(options) {
		// Add All option
		var entry = new Object();
		entry.label = this.messages.msgAll;
		entry.value = this._staticCommon._PREFERENCE_OPTION_VALUE_ALL;
		options.unshift(entry);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Object a
 	 * @param Object b
 	 * @return Object
 	 */		
	_sortPreferencesArray: function(a, b) {
		if (a.label < b.label) {
			// Less than
			return -1;
		} else if (a.label > b.label) {
			// Greater than
			return 1;
		}
		// Equal
		return 0;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @return void
 	 */
	_getReport: function(parameterNames, parameterValues) {
		// Generate the hash key for the report request
		var hashKey = this._hashArray(parameterValues);
		// Check to see if the report is cached
		var cachedHtml = this._getReportFromCache(hashKey);
		var isReportCached = (cachedHtml != null);
		// Display whether the report is cached or live
		this._displayCacheStatus(isReportCached);
		// Display cached report
		if (isReportCached) {
			// Add the report to the display area
			this._addReportToViewlet(cachedHtml);
			// Service invocation unnecessary
			this.getSite().doneRefresh();
			return;
		}
		// Generate the click-through URL
		this._generateClickThroughURL(
			this._reportUUID, 
			parameterNames, 
			parameterValues
		);
		
		// Update loading message
		this._updateLoadingMessage(this.messages.msgLoading);
		// Show the loading message
		this._showLoadingArea();
		
		// Remember the zoom state at the time of this request
		var previousZoomState = this.isZoomed();
		
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(html) {
				self._processReportHTML(html, hashKey, previousZoomState);
				self.getSite().doneRefresh();
			},
			// Service failure
			failure: function(errorObject) {
				self._logError(errorObject);
				self._displayErrorMessage(self.messages.msgReportLoadingFailed, errorObject);
				self.getSite().doneRefresh();
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service
		this._callRESTGetRenderedReport(
			handler, 
			this._reportUUID,
			parameterNames, 
			parameterValues);

	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Boolean cached
 	 * @return void
 	 */	
	_displayCacheStatus: function(cached) {
		// Only show status in debug mode
		if (djConfig.isDebug == false) {
			return;
		}
		// Determine content
		var content = null;
		if (cached) {
			content = "Cached";
		} else {
			content = "Live";
		}		
		
		// Get the existing cache status indicator (if any)
		var cacheStatus = dojo.byId(this._getCacheStatusDomID());				
		
		if (cacheStatus == null) {
			// Create the container
			var container = document.createElement("span");
			container.setAttribute("id", this._getCacheStatusDomID());
			dojo.addClass(container, "cache-status");
			container.innerHTML = content;		
			var node = this._displayArea;
			node.appendChild(container);
		} else {
			// Reuse the container
			cacheStatus.innerHTML = content;
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @return void
 	 */	
	_displayParameters: function(parent, parameterNames, parameterValues) {
		// Table
		var container = document.createElement("table");		
		dojo.addClass(container, "parameters");
		var body = document.createElement("tbody");
		
		for (var i = 0; i < parameterNames.length; i++) {
			// Row
			var row = document.createElement("tr");		
			// Column 1
			var col1 = document.createElement("td");
			dojo.addClass(col1, "label");
			col1.innerHTML = parameterNames[i] + ":";
			// Column 2
			var col2 = document.createElement("td");
			dojo.addClass(col2, "value");
			col2.innerHTML = parameterValues[i];		
			// Add columns to row
			row.appendChild(col1);
			row.appendChild(col2);
			// Add row to table
			body.appendChild(row);
		}
		
		// Assemble contents
		container.appendChild(body);
		parent.appendChild(container);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * @return void
 	 */	
	_addReportToViewlet: function(html) {
		// Update loading message
		this._updateLoadingMessage("");
		// Get the attach point
		var node = this._displayArea;
		// Add the report HTML to the node
		this._addHTMLToNode(html, node);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * @param Element node
 	 * @return void
 	 */	
	_addHTMLToNode: function(html, node) {
		// Create a wrapping div for the parameters and report nodes
		var container = document.createElement("div");
		container.setAttribute("id", this._getReportDomID());	
		
		// Add the parameters node (current preference values)
		this._addParametersToNode(container);

		// Create the report node
		var reportNode = document.createElement("div");
		// Set the content to be the html
		reportNode.innerHTML = html;
		// Add the report node
		container.appendChild(reportNode);
		
		// Add the wrapping div to the node
		this._addReportNode(node, container);
		// Show the display area
		this._showDisplayArea();		
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return String
 	 */	
	_getReportDomID: function() {
		return this._staticCommon._REPORT_DOM_ID + "_" + this._viewletID;	
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return String
 	 */	
	_getCacheStatusDomID: function() {
		return this._staticCommon._CACHE_STATUS_DOM_ID + "_" + this._viewletID;	
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element targetParent
 	 * @param Element newReportNode
 	 * @return void
 	 */		
	_addReportNode: function(targetParent, newReportNode) {
		// Get the old report node (if any)
		var oldReportNode = dojo.byId(this._getReportDomID());		
		
		if (oldReportNode == null) {
			targetParent.appendChild(newReportNode);
		} else {
			targetParent.replaceChild(newReportNode, oldReportNode);
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String reportUUID
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @return void
 	 */	
	_generateClickThroughURL: function(reportUUID, parameterNames, parameterValues) {
		// Get the corresponding normal report UUID for the specified micro report UUID
		var normalReportUUID = this._staticCommon._microReportIDMap[reportUUID];
		// Ensure we have a value
		try {
			this._isStringDefined(normalReportUUID, true, "normalReportUUID");
		} catch(errorObject) {
			this._logError(errorObject);
			// BUG: Error message gets overidden in viewlet content area
			this._displayErrorMessage(dojo.string.substitute(this.messages.msgNoMatchingNormalReport, [reportUUID]), errorObject);
			return;
		}		
		
		// Ensure we the number of names and values matches
		if (parameterNames.length != parameterValues.length) {
			var errorObject = new Error(
				dojo.string.substitute(this.messages.msgWrongNumberOfEntries, [parameterNames.length, parameterValues.length])
			);			
			this._logError(errorObject);
			// BUG: Error message gets overidden in viewlet content area
			this._displayErrorMessage(this.messages.msgURLGenerationFailed, errorObject);
			return;
		}
		
		// Format: #action=com.ibm.team.reports.viewReport&reportUUID=workitems.OpenWorkItemsByType&parameterName=ProjectAreaName&parameterName=TeamAreaName&parameterName=Category&parameterName=Interval&parameterName=Zoom&parameterValue='Jazz%20Project'&parameterValue='%2FJazz%2FDashboard'&parameterValue='Dashboards'&parameterValue='0.6%20M4D1'&parameterValue=false
		// Assemble URL
		var url = this._staticCommon._URI_REPORT_ACTION + 
					normalReportUUID;

		// Determine the number of parameters to process
		var parameterCount;
		if (parameterNames[parameterNames.length - 1] === this._staticCommon._PARAMETER_NAME_ZOOM) {
			// Skip "Zoom" parameter (always last parameter if specified)
			parameterCount = parameterNames.length - 1;
		} else {
			// Process all parameters
			parameterCount = parameterNames.length;
		}

		// Serialize parameter names
		for (var i = 0; i < parameterCount; i++) {
			// Get the current parameter and encode it
			var parameterName = encodeURIComponent(parameterNames[i]);
			// Append the name value pair
			url = url + this._staticCommon._URI_REPORT_PARAMETER_NAME + parameterName;
		}
		// Serialize parameter values
		for (var i = 0; i < parameterCount; i++) {
			// Get the current parameter and encode it
			var parameterValue = encodeURIComponent(parameterValues[i]);
			// Append the name value pair
			url = url + this._staticCommon._URI_REPORT_PARAMETER_VALUE + parameterValue;
		}
		// Set global
		this._clickThroughURL = url;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * @param String hashkey
 	 * @param boolean previousZoomState
 	 * @return boolean
 	 */	
	_processReportHTML: function(html, hashKey, previousZoomState) {
		// Check if a report was found
		// TODO: MP: Need to update to scan for tables for frequent test failures / regression reports
		
		if (previousZoomState != this.isZoomed()) {
			// NO-OP
			// If the zoom state made at request time is different than 
			// the zoom state on request completion (now), do not add the 
			// report (but still cache it)
		} else if (this._containsReport(html)) {
			// Report found
			if (this._isExpressEdition) {
				// No Reports Web UI vertical in Express editions
				// Disable chart drill-in
				// Note: Only applicable to Build Duration report (at the 
				// moment)
				html = this._removeHrefs(html);
			}				
			// Add the report to the display area
			this._addReportToViewlet(html);
		} else {
			// Report NOT found
			html = "<div class=\"no-results\">" + this._messageNoResults + "</div>";
			this._addReportToViewlet(html);
		}
		
		// Cache the report
		this._putReportInCache(hashKey, html);		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * 
 	 * @return String
 	 */	
	_removeHrefs: function(html) {
		// Before
		// <area title="4.838" target="_self" href="#action=com.ibm.team.reports..." coords="91,139,91,206,100,206,100,139" shape="poly"/>	
		// After
		// <area title="4.838" target="_self" coords="91,139,91,206,100,206,100,139" shape="poly"/>		

		var result = html.replace(/href=\".*?\"/g, "");

		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * @return boolean
 	 */	
	_containsReport: function(html) {
		var matcher = new String(html);
		if (matcher.match(/\.png/)) {
			return true;
		}
		return false;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return boolean
 	 */	
	_isSpecialValue: function(value) {
		var matcher = new String(value);
		if (matcher.match(/^\{/)) {
			return true;
		}
		return false;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @return void
 	 */	
	_callRESTGetRenderedReport: function(handler, reportUUID, parameterNames, parameterValues) {
		// Create the arguments
		var arguments = {
			projectAreaUUID: getCurrentProjectArea().itemId,
			reportUUID: reportUUID,
			parameterName: parameterNames,
			parameterValue: parameterValues,
			showArchived: false,
			webui: true
		};
		
		this._callRESTReportsService(handler, "getRenderedReport", arguments);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String parameterName 
 	 * @return void
 	 */	
	_callRESTGetParameterValues: function(handler, reportUUID, parameterName) {
		// Create the arguments
		var arguments = {
			reportUUID: reportUUID,
			parameter: parameterName,
			showArchived: false,
			projectAreaUUID: getCurrentProjectArea().itemId
		};
		this._callRESTReportsService(handler, "getParameterValues", arguments);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String groupName 
 	 * @return void
 	 */	
	_callRESTGetParameterGroupValues: function(handler, reportUUID, groupName) {
		// Create the arguments
		var arguments = {
			reportUUID: reportUUID,
			groupName: groupName,
			showArchived: false,
			projectAreaUUID: getCurrentProjectArea().itemId
		};
		this._callRESTReportsService(handler, "getParameterGroupValues", arguments);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String method
 	 * @param Object arguments
 	 * @return void
 	 */	
	_callRESTReportsService: function(handler, method, arguments) {
		// Create the service request
		var request = new ServiceRequest(
					this._staticCommon._REPORT_REST_SERVICE_URI,
					method,
					arguments);
		this.getServiceTracker().register(request, handler);
		// Invoke the service
		TeamServerClient.invokeService(request, handler);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * 		. String parameterName
	 *		. Array labels : String
	 *		. Array values : String
 	 * @return Object
	 *			. Array labels : String
	 *			. Array values : String
 	 * @throws Error
 	 */	
	_processDTOSingleParameters: function(dto) {
		// Note:  Do not need to process parameterName

		// Validation
		// DTO
		this._isObjectDefined(dto, true, "dto");
		// Values
		this._hasProperty(dto, "values", true, "dto");
		this._isArray(dto.values, true, "values");
		// Labels
		this._hasProperty(dto, "labels", true, "dto");
		this._isArray(dto.labels, true, "labels");

		var result = new Object();
		result.labels = dto.labels;
		result.values = dto.values;
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param com.ibm.team.reports.rest:ReportParameterGroupDTO dto
 	 * 		. String groupName
	 *		. Array parameters : Object
	 *			. Array labels : String
	 *			. Array values : String
	 *			. String parameterName (1)
	 *			. Number flags
	 *			. Array children : Object
	 *				. Array labels : String
	 *				. Array values : String
	 *				. String parameterName (2)
	 *				. Number flags
	 *				. Array children : Object
	 *					. Array labels : String
	 *					. Array values : String
	 *					. String parameterName (3)
	 *					. Number flags
	 *					. Array children : Object
 	 * @return Array : [String]=Object
 	 * 				. Array values : String
 	 * 				. Array map : [String]=Object
 	 * 					. Array values : String
 	 * 					. Array map : [String]=Object
 	 * 						etc.			
 	 * @throws Error
 	 */	
	_processDTOGroupParameters: function(dto) {
		// Note:  Do not need to process groupName

		// Validation
		// DTO
		this._isObjectDefined(dto, true, "dto");
		// Parameters
		this._hasProperty(dto, "parameters", true, "dto");
		this._isArray(dto.parameters, true, "parameters");
		// Only one entry expected
		if (dto.parameters.length != 1) {
			throw new Error(this.messages.msgNoParametersDefined);
		}			
		// First parameter
		this._isObjectDefined(dto.parameters[0], true, "dto.parameters[0]");
		// Values		
		this._hasProperty(dto.parameters[0], "values", true, "dto.parameters[0]");
		this._isArray(dto.parameters[0].values, true, "values");

		// Ensure we have project areas
		if (dto.parameters[0].values.length == 0) {
			throw new Error(this.messages.msgNoProjectAreasDefined);
		}			
		
		// Always process and use the {Current Project Area} whenever possible
		// Otherwise use the first project area
		var projectAreaIndex = 0;
		var projectAreaValue = dto.parameters[0].values[projectAreaIndex];
		for (var index = 0; index < dto.parameters[0].values.length; index++) {
			var currentValue = dto.parameters[0].values[index];
			if (currentValue == this._staticCommon._SPECIAL_PROJECT_AREA) {
				projectAreaIndex = index;
				projectAreaValue = this._staticCommon._SPECIAL_PROJECT_AREA;
				break;
			}
		}
		
		// Create map
		var map = new Array();
		// Validate project area value
		this._isStringDefined(projectAreaValue, true, "projectAreaValue");
		// Index object			
		map[projectAreaValue] = new Object();
		
		// Validation
		// Children		
		if (this._hasProperty(dto.parameters[0], "children", false, "dto.parameters[0]") === false) {
			return map;
		}		
		this._isArray(dto.parameters[0].children, true, "children");

		// Ensure that the number of values equals the number of children
		if (dto.parameters[0].values.length != dto.parameters[0].children.length) {
			throw new Error(
				dojo.string.substitute(this.messages.msgWrongNumberOfProjectAreas, [dto.parameters[0].values.length, dto.parameters[0].children.length])
			);
		}			
	
		// Process child
		var child = dto.parameters[0].children[projectAreaIndex];
		this._processDTOGroupParametersStep(child, map[projectAreaValue]);
		
		return map;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object child
 	 * 			. Array labels : String
	 *			. Array values : String
	 *			. String parameterName
	 *			. Number flags
	 *			. Array children : Object
	 * @param Object parent
 	 * @return void
 	 * 			
 	 * @throws Error
 	 */	
	_processDTOGroupParametersStep: function(child, parent) {
		// Note:  Do not need to process labels
		// Note:  Do not need to process flags
		// Note:  Do not need to process parameterName
		
		// Validation
		// DTO
		this._isObjectDefined(child, true, "child");
		// Values		
		if (this._hasProperty(child, "values", false, "child") == false) {
			// TODO: What to populate in array?
			return;
		}
		this._isArray(child.values, true, "values");
		
		// Create map
		var map = new Array();
		// Create filtered values container
		var filteredValues = new Array();
		// Process values
		for (var i = 0; i < child.values.length; i++) {
			var value = child.values[i];
			// Skip undefined values
			if (this._isStringDefined(value, false, "value") == false) {
				continue;
			}
			// Index object			
			var object = new Object();
			map[value] = object;
			filteredValues.push(value);
			// TODO: MP: Need to handle labels in the future?
		}
		// Add values to parent
		parent.values = filteredValues;
		// Add map to parent
		parent.map = map;
		
		// Validation
		// Children		
		if (this._hasProperty(child, "children", false, "child") == false) {
			// TODO: What to populate in array?
			return;
		}
		this._isArray(child.children, true, "children");
		
		// Ensure that the number of values equals the number of children
		if (child.values.length != child.children.length) {
			throw new Error(
				dojo.string.substitute(this.messages.msgWrongNumberOfParameters, [child.values.length, child.children.length])
			);
		}		
		
		// Process children
		for (var i = 0; i < child.children.length; i++) {
			var grandchild = child.children[i];
			// Skip undefined granchildren
			if (this._isObjectDefined(grandchild, false, "grandchild") == false) {
				continue;
			}
			var value = child.values[i];
			this._processDTOGroupParametersStep(grandchild, map[value]);
		}
		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String groupName 
 	 * @param Function successCallBack
 	 * @param Function errorCallBack
 	 * @return void
 	 */
	_getGroupParameters: function(groupName, successCallBack, errorCallBack) {
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(dto) {
				try {
					successCallBack(self, groupName, dto);
				} catch(errorObject) {
					errorCallBack(self, groupName);
					self._logError(errorObject);
					self._displayErrorMessage(self.messages.msgParameterLoadingFailed, errorObject);					
				}
			},
			// Service failure
			failure: function(errorObject) {
				errorCallBack(self, groupName);
				self._logError(errorObject);
				self._displayErrorMessage(self.messages.msgParameterLoadingFailed, errorObject);					
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service
		this._callRESTGetParameterGroupValues(
			handler, 
			this._reportUUID, 
			groupName);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String parameterName 
 	 * @param Function successCallBack
 	 * @param Function errorCallBack
 	 * @return void
 	 */
	_getSingleParameters: function(parameterName, successCallBack, errorCallBack) {
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(dto) {
				try {
					successCallBack(self, parameterName, dto);
				} catch(errorObject) {
					errorCallBack(self, parameterName);
					self._logError(errorObject);
					self._displayErrorMessage(self.messages.msgParameterLoadingFailed, errorObject);					
				}
			},
			// Service failure
			failure: function(errorObject) {
				errorCallBack(self, parameterName);
				self._logError(errorObject);
				self._displayErrorMessage(self.messages.msgParameterLoadingFailed, errorObject);					
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service
		this._callRESTGetParameterValues(
			handler, 
			this._reportUUID, 
			parameterName);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizePreferenceMultiValue: function(value) {
		return this._normalizeMultiValue(value, "_normalizePreferenceSingleValue");
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @param String normalizeValueMethodName
 	 * 
 	 * @return String
 	 */	
	_normalizeMultiValue: function(value, normalizeValueMethodName) {
		if (value == this._staticCommon._PREFERENCE_OPTION_VALUE_ALL) {
			return this[normalizeValueMethodName](this.messages.msgAll);
		}
		// Parse values		
		var parsedValues = value.split(this._staticCommon._MULTI_PARAMETER_SEPARATOR);
		
		if (parsedValues.length < 1) {
			// Error
			var errorObject = new Error(dojo.string.substitute(this.messages.msgPrefValueParsingFailed, [value]));
			this._logError(errorObject);
			return this[normalizeValueMethodName](value);			
		} else if (parsedValues.length == 1) {
			// TODO: Should do index of separator before parse?
			// Single value
			return this[normalizeValueMethodName](value);
		}
		// Multiple values (> 1)
		var result = this[normalizeValueMethodName](parsedValues[0]);
		for (var index = 1; index < parsedValues.length; index++) {
			result = result + 
					this._staticCommon._MULTI_PARAMETER_SEPARATOR + 
					" " +
					this[normalizeValueMethodName](parsedValues[index]);
		}
		return result;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizePreferenceSingleValue: function(value) {
		if (value === this._staticCommon._SPECIAL_MY_TEAM_AREAS) {
			return this.messages.msgMyTeamAreas;
		} else if (value === this._staticCommon._SPECIAL_CURRENT_ITERATION) {
			return this.messages.msgCurrentIteration;
		}
		return value;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeTeamAreaPreferenceSingleValue: function(value) {

		if (value.charAt(0) === "{") {
			// Special variable
			return this._normalizeSpecialVariable1(value);
		}

		var delimeter = "/";
		var lastDelimeterIndex = value.lastIndexOf(delimeter);
		
		if (lastDelimeterIndex < 0) {
			// No delimeters
			// e.g. Jazz
			// Return as-is (this should not happen)
			// e.g. Jazz
			return value;
		} else if (lastDelimeterIndex === 0) {
			// Delimeter only at first position
			// e.g. /Jazz Development
			// Remove delimeter
			// e.g. Jazz Development
			return value.substr(1);
		}
		// More than one delimeter found
		// e.g. /Jazz Development/Incubator/Code Coverage
		// Extract qualifier
		// e.g. /Jazz Development/Incubator
		var valueQualifier = value.substr(0, lastDelimeterIndex);
		// Normalize qualifier by removing the delimeter at the first position
		// and space out all other delimeters
		// e.g. Jazz Development / Incubator
		var normalizedValueQualifier = valueQualifier.substr(1).replace(delimeter, " / ");
		// Get the team area name
		// e.g. Code Coverage
		var valueLocalName = value.substr(lastDelimeterIndex + 1);
		// Format:  Code Coverage - Jazz Development / Incubator
		return valueLocalName + 
			   "<span class=\"label-qualifier\">" +
			   " - " +
			   normalizedValueQualifier +
			   "</span>";
	},

	/*
 	 * Private Method
 	 * 
 	 * Used for multi-selectable intervals
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeIntervalPreferenceSingleValue: function(value) {
		if (value.charAt(0) === "{") {
			// Special variable
			return this._normalizeSpecialVariable1(value);
		}
		return value;
	},

	/*
 	 * Private Method
 	 * 
 	 * Used for singly-selectable intervals
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeIntervalPreferenceSingleValue2: function(value) {
		if (value.charAt(0) === "{") {
			// Special variable
			return this._normalizeSpecialVariable2(value);
		}
		return value;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * 
 	 * @return String
 	 */	
	_normalizeContentAreaSingleTeam: function(value) {
		// Normalize the dashboard name
		var result = this._normalizeDashboardName(value);
		
		if (this._isExpressEdition) {
			// No team dashboards in Express editions
			// Disable dashboard links
			return result.displayName;
		}		
		return this._createDashboardDrillInLink(result);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object result
 	 * 				. String drillInName
 	 * 				. String displayName
 	 * 				. Number type
 	 * 					0: All
 	 * 					1: Project
 	 * 					2: Team
 	 * 
 	 * @return String
 	 */	
	_createDashboardDrillInLink: function(result) {
		
		// Assemble the URL
		var url;
				  
		if (result.type === 0) {
			// All
			url = this._staticCommon._DASHBOARD_DRILL_ACTION_VIEW_ALL;
		} else if (result.type === 1) {
			// Project
			url = this._staticCommon._DASHBOARD_DRILL_ACTION_VIEW;
		} else {
			// Team
			url = this._staticCommon._DASHBOARD_DRILL_ACTION_VIEW +
				  this._staticCommon._DASHBOARD_DRILL_IN_PARAM_TEAM +
				  result.drillInName;
		}
	  
		var link = "<a title=\"" +
				   result.displayName +
				   "\" href=\"" +
				   url +
				   "\">" +
				   result.displayName +
				   "</a>";
		
		return link;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String dashboardName
 	 * 
 	 * @return Object
 	 * 				. String drillInName
 	 * 				. String displayName
 	 * 				. Number type
 	 * 					0: All
 	 * 					1: Project
 	 * 					2: Team
  	 */	
	_normalizeDashboardName: function(dashboardName) {
		
		var result = new Object();
		
		if (dashboardName === this.messages.msgAll) {
			// All
			result.drillInName = null;
			result.displayName = dashboardName;
			result.type = 0;
			return result;
		} else if (dashboardName === this._staticCommon._SPECIAL_MY_TEAM_AREAS) {
			// Special variable
			result.drillInName = null; 
			result.displayName = this.messages.msgMyTeamAreas;
			// Handle as All (no mapping between special variables and Dashboards)
			result.type = 0;
			return result;
		}
		
		// Replace the first forward slash (if any)
		// Before: /Jazz/Dashboard
		// After:  Jazz/Dashboard
		var splittableDashboardName = dashboardName.replace(/^\//, "");
		// Split the name into component parts
		var delimeter = "/";
		var nameComponents = splittableDashboardName.split(delimeter);
		
		if (nameComponents.length === 0) {
			// Error - This should never happen
			result.drillInName = null;
			result.displayName = dashboardName;
			result.type = 0;
			return result;
		} else if ((nameComponents.length === 1) &&
			(decodeURIComponent(this._getProjectAreaName()) === nameComponents[0])) {
			// Project
			result.drillInName = null;
			result.displayName = dashboardName;
			result.type = 1;
			return result;
		}
		// Name components >= 1
		
		// Encode component parts
		// Before: Jazz Project Team
		// After:  Jazz%20Project%20Team
		var normalizedDashboardName = encodeURIComponent(nameComponents[0]);
		for (var index = 1; index < nameComponents.length; index++) {
			// e.g. Jazz/Incubator/Component%20Development
			normalizedDashboardName = normalizedDashboardName +
									  delimeter +
									  encodeURIComponent(nameComponents[index]);
		}		
		
		result.drillInName = normalizedDashboardName;
		result.displayName = dashboardName;
		result.type = 2;
		return result;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getProjectAreaName: function() {
		// Use precomputed value if defined
		if (this._isStringDefined(this._projectAreaName, false, "_projectAreaName")) {
			return this._projectAreaName;
		}
		
		// Parse the current URL for the project area name
		var text = new String(document.location.href);
		// e.g. https://localhost:9443/jazz/web/projects/Eclipse%20Project?debug=true#action=jazz.viewPage&id=com.ibm.team.dashboard
		var pattern = /\/projects\/([a-zA-Z0-9%_-]+)[\?|#]/;
		var result = text.match(pattern);
		if ((result == null) || 
			(result.length < 2)) {
			this._projectAreaName = "Unknown%20Project";
		} else {
			// First result entry is the whole pattern including the match
			// Second result entry is just the match
			// e.g. Eclipse%20Project
			this._projectAreaName = result[1];
		} 
		
		return this._projectAreaName;		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object parameterObject
 	 * 					. Array map
 	 * 						. Object 1
 	 * 						. Object 2
 	 * 						...
 	 * 						. Object N
 	 * 
 	 * @return Array : [Number] = Object
 	 */	
	_getAllParameterChildren: function(parameterObject) {
		var list = new Array();
		for (var propertyName in parameterObject.map) {
			list.push(parameterObject.map[propertyName]);
		}
		return list;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object parameters
 	 * @return Number
 	 * 			 1: Loaded
 	 * 			 0: Not Loaded
 	 * 			-1: Error
 	 */	
	_isParametersLoaded: function(parameters) {
		if (parameters === null) {
			return 0;
		} else if (parameters === this._staticCommon._DTO_PARAMETERS_FAILURE) {
			return -1;
		}
		return 1;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return Array : String
 	 */	
	_parsePreferenceMultiValue: function(value) {
		
		if (this._isStringDefined(value, false, "value") == false) {
			return new Array();
		}
		// Parse values		
		var parsedValues = value.split(this._staticCommon._MULTI_PARAMETER_SEPARATOR);
		return parsedValues;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeSpecialVariable1: function(value) {
		// UI Presentation:  Swap list entry
		var result = this._normalizePreferenceSingleValue(value);
		// Prepend with non-breaking whitespace character in order to place
		// the special variable first in terms of sort order
		// e.g. ASCII:  & < 0 < A < a
		return "&#8203;" + 
			   "<span class=\"special-variable\">" +
			   result +
			   "</span>";
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeSpecialVariable2: function(value) {
		// UI Presentation:  Drop menu entry
		var result = "- " + 
					 this._normalizePreferenceSingleValue(value) +
					 " -";
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return Array : [String]=boolean
 	 */	
	_parsePreferenceMultiValueMap: function(value) {
		
		if (this._isStringDefined(value, false, "value") == false) {
			return new Array();
		}
		// Parse values		
		var parsedValues = value.split(this._staticCommon._MULTI_PARAMETER_SEPARATOR);
		// Index values
		var map = new Array();
		for (var index = 0; index < parsedValues.length; index++) {
			var value = parsedValues[index];
			map[value] = true;
		}
		
		return map;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeParameterMultiValue: function(value) {
		if (value == this._staticCommon._PREFERENCE_OPTION_VALUE_ALL) {
			return this._normalizeParameterSingleValue("");
		}
		// Parse values		
		var parsedValues = value.split(this._staticCommon._MULTI_PARAMETER_SEPARATOR);
		
		if (parsedValues.length < 1) {
			// Error
			var errorObject = new Error(dojo.string.substitute(this.messages.msgParamValueParsingFailed, [value]));
			this._logError(errorObject);
			return this._normalizeParameterSingleValue(value);			
		} else if (parsedValues.length == 1) {
			// TODO: Should do index of separator before parse?
			// Single value
			return this._normalizeParameterSingleValue(value);
		}
		// Multiple values (> 1)
		var result = this._normalizeParameterSingleValue(parsedValues[0]);
		for (var index = 1; index < parsedValues.length; index++) {
			result = result + 
					this._staticCommon._MULTI_PARAMETER_SEPARATOR + 
					this._normalizeParameterSingleValue(parsedValues[index]);
		}
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeParameterSingleValue: function(value) {
		return this._quote(value);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_quote: function(value) {
		return "'" + value + "'";
	},	

	// Utility Methods -------------------------------------------------------->

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventPrefLinkClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Load the settings area
		this.getSite().toggleSettings();
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String message
 	 * @return void
 	 */	
	_displayMessage: function(message) {
		// Get attach point
		var node = this._displayArea;
		
		// Create a wrapping div 
		var container = document.createElement("div");
		container.setAttribute("id", this._getReportDomID());
		// Create a text node for the message
		var textNode = document.createTextNode(message);
		// Assemble contents		
		container.appendChild(textNode);
		// Add the wrapping div to the node
		this._addReportNode(node, container);		
		
		// Show the display area
		this._showDisplayArea();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return void
 	 */	
	_updateLoadingMessage: function(text) {
		// Get the attach point
		var node = this._loadingArea;
		// Get the text node
		var textNode = node.firstChild;
		
		if ((textNode == null) &&
			(text.length == 0)) {
			// Nothing to do
		} else if (text.length == 0) {
			// Remove the old text node
			node.removeChild(textNode);
		} else if (textNode == null) {
			// Create a new text node
			textNode = document.createTextNode(text);
			node.appendChild(textNode);
		} else {
			// Modify the existing text node
			textNode.replaceData(0, textNode.length, text); 
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String message
 	 * @param Error errorObject
 	 * @return void
 	 */	
	_displayErrorMessage: function(message, errorObject) {
		var text = dojo.string.substitute(this.messages.msgError, [message]);
		var errorMessage = errorObject.message;
		// Add the error message
		if ((typeof(errorMessage) == "string") &&
			(errorMessage.length != 0)) {
			text = text + 
					": " +
					errorMessage;
		} 
		this._displayMessage(text);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_displayMsgSelectPreferences: function() {
		// Get attach point
		var node = this._displayArea;
		// Create a wrapping div 
		var container = document.createElement("div");
		container.setAttribute("id", this._getReportDomID());	
		
		// Create actionable status message
		var text = this.messages.msgActionConfigureReport;
		var links = I18nUtil.addMessage(container, text);
		var link = links[0];
		link.href = "javascript:void(0);";
		this.connect(link, "onclick", this._handleEventPrefLinkClicked);		
		
		// Add the wrapping div to the node
		this._addReportNode(node, container);
		// Show the display area
		this._showDisplayArea();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showDisplayArea: function() {
		// Show the loading area
		var loadingNode = this._loadingArea;
		this._updateVisibility(loadingNode, false);
		// Hide the loading image
		this.getSite().setLoading(false);
		// Hide the display area
		var displayNode = this._displayArea;
		this._updateVisibility(displayNode, true);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showLoadingArea: function() {
		// Show the loading area
		var loadingNode = this._loadingArea;
		this._updateVisibility(loadingNode, true);
		// Show the loading image
		this.getSite().setLoading(true);
		// Hide the display area
		var displayNode = this._displayArea;
		this._updateVisibility(displayNode, false);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show == true) {
			// Show
			node.style.display = "block";
		} else {
			// Hide
			node.style.display = "none";
		}
	},			

	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param String property
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 */	
	_hasProperty: function(variable, property, throwError, variableName) {
		// Check to see if the property is in the variable
		if (property in variable) {
			return true;
		}
		if (throwError) {
			throw new Error(dojo.string.substitute(this.messages.msgPropertyDoesNotExist, [variableName, property]));
		}
		return false;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 */	
	_isArray: function(variable, throwError, variableName) {
		// Check to see if the variable is an array
		if (variable instanceof Array) {
			return true;
		}
		if (throwError) {
			throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotAnArray, [variableName]));
		}
		return false;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isStringDefined: function(text, throwError, variableName) {
		// Check if the text is null (with casting)
		if (text == null) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNull, [variableName]));
			}
			return false;
		}		
		// Check if the text is a String
	    if (typeof(text) != 'string') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotAString, [variableName]));
			}
			return false;
		}
		// Check if the text has content
		if (text.length == 0) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsZeroLength, [variableName]));
			}
			return false;
		}		
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Number number
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isNumberDefined: function(number, throwError, variableName) {
		// Check if the number is a Number
	    if (typeof(number) != 'number') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotANumber, [variableName]));
			}
			return false;
		}
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param boolean bool
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isBooleanDefined: function(bool, throwError, variableName) {
		// Check if the boolean is a boolean
	    if (typeof(bool) != 'boolean') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgNotValidBooleanType, [variableName]));
			}
			return false;
		}
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isObjectDefined: function(variable, throwError, variableName) {
		// Check if the variable exists (Any scope)
	    if (typeof(variable) != 'object') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgNotValidObjectType, [variableName]));
			}
			return false;
		}
		// Check if the variable is not null (with casting)
		if (variable == null) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNull, [variableName]));
			}
			return false;
		}
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Error
 	 * @return void
 	 */
	_logError: function(errorObject) {
		
		var errorMessage = "[ERROR]: " + 
							this.toString() + 
							": " + 
							errorObject.message;
		console.error(errorMessage);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Node node
 	 * @return void
 	 */	
	_removeAllChildren: function(node) {
		// Remove all child nodes if any are defined
		while (node.firstChild != null) {
			node.removeChild(node.firstChild);
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return String
 	 */		
	 _hashString: function(text) {
		var hash = text.length;
		for(var i = 0; i < text.length; i++) {
			hash = ((hash << 5) ^ (hash >> 27)) ^ text.charCodeAt(i);
		}
		var hashKeyString = "K" + new Number(hash).toString();
		
		return hashKeyString;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Array list
 	 * @return String
 	 */		
	 _hashArray: function(list) {
	 	var text = new String();
		for (var i = 0; i < list.length; i++) {
			text = text + list[i];
		}
		return this._hashString(text);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String key
 	 * @param String value
 	 * @return void
 	 */		
	 _putReportInCache: function(key, value) {
	 	
		var stack = this._reportKeyStack;
		var cache = this._reportCache;
		
		// Check to see if the key already exists in the cache
		if (key in cache) {
			// The key exists
			// Replace the old value with the new value
			cache[key] = value;
			// TODO: MP: Move key to end of stack to adjust priority
			return;
		}
		// The key does not exist in the cache
		
		// Check to see if the cache is full
		if (stack.length == this._staticCommon._REPORT_CACHE_SIZE) {
			// Cache is full
			// Remove the earliest report cached from the priority stack
			// to make way for the new report
			var oldKey = stack.shift();
			// Remove the earliest report from the cache
			delete(cache[oldKey]);
		}
		// Add the new report to the priority stack
		stack.push(key);
		// Add the new report to the cache
		cache[key] = value;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Array map : [String]=Object
 	 * 					. Array values : String
 	 * 					. Array map : [String]=Object
 	 * 						. Array values : String
 	 * 						. Array map : [String]=Object
 	 * 							etc.		
 	 * @param String indent
 	 * @param Number level
 	 * @return void
 	 */		
	 _debugCascadingParameters: function(map, indent, level) {
	 	
		console.debug(indent + "Level " + level + ":");
		indent = indent + "   ";
		level++;
		
		for (var name in map) {
			// Process values
			if (("values" in map[name]) == false) {
				continue;
			}
			for (var i = 0; i < map[name].values.length; i++) {
				console.debug(indent + map[name].values[i]);
			}
			console.debug(indent + "---");
		}

		for (var name in map) {
			// Process map
			if (("map" in map[name]) == false) {
				continue;
			}
			this._debugCascadingParameters(map[name].map, indent, level);
		}

		
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String key
 	 * @return String
 	 */		
	 _debugCache: function(key) {
	 	
		var stack = this._reportKeyStack;
		var cache = this._reportCache;
		var cacheEntries = new String();
		
		for (var x in cache) {
			cacheEntries = cacheEntries + x + ", ";
		}
		console.debug(cacheEntries);
		console.debug(stack);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String key
 	 * @return String
 	 */		
	_getReportFromCache: function(key) {
	 	
		var cache = this._reportCache;
		// Check to see if the key already exists in the cache
		if (key in cache) {
			// The key exists in the cache
			return cache[key];
		}
		// The key does not exist in the cache
		return null;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @param String label
 	 * @return Number
 	 */		
	_getIconType: function(value, label) {
		return 0; 	
	},	

	// Class Methods ---------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._staticCommon._CLASSNAME;
	}	
	
});


})();

