/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs");

dojo.require("dojo.i18n"); 
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableManager");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dojo.dnd.autoscroll");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.Target");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.client.internal.ColumnHelper");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletPageHelper");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.require("dijit.Menu");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");


(function() {
	var draggableManager = com.ibm.team.dashboard.web.ui.internal.dnd.draggableManager;
	var Target = com.ibm.team.dashboard.web.ui.internal.dnd.Target;
	var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
	var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
	var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
	var ColumnHelper = com.ibm.team.dashboard.web.client.internal.ColumnHelper;
	var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
	var Util = com.ibm.team.dashboard.web.util.internal.Util;
	var ViewletPageHelper = com.ibm.team.dashboard.web.client.internal.ViewletPageHelper;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var messages;
	
	dojo.declare("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs", com.ibm.team.dashboard.web.ui.internal.dnd.DraggableContainer, {
		
		//constrains movement of drag avatar to the area of its source container ("A").
		constrainAvatar: "A",
		
		//Adjust avatar position and drop area tolerances
		WIDGET_TOP_MARGIN: 0,
		WIDGET_BOTTOM_MARGIN: 2,
		WIDGET_WIDTH_PADDING: 4,
		WIDGET_LEFT_MARGIN: 4,
		WIDGET_RIGHT_MARGIN: 4,
		
		constructor: function(node, params){
			this.avatarClass = "draggableTabsAvatar";
			dojo.addClass(node,"draggableTabs");
			this._disabled = false;
			this.horizontal = true;
			this.parent = params.parent;
			this.dashboardItem = params.dashboardItem;
			this.dashboardContainer = params.dashboardContainer;
			this.selectedTabId = params.selectedTabId;	
			this.tabTargets = new Array();
			if (params.disabled) {
				this.setEditMode(!params.disabled);
			}
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		},
		add: function(widget, params) {
			//always ignore widget parameter - replace with new object
			widget = new Object();
			widget.domNode = document.createElement("div");
			widget.domNode.className = "tab";
			var innerDiv = document.createElement("div");
			innerDiv.className = "tabInnerDiv";
			var tabTitle = document.createElement("a");
			tabTitle.className = "tabTitle";
			tabTitle.innerHTML = HTMLUtility.escapeHTML(params.title);
			innerDiv.appendChild(tabTitle);
			widget.domNode.appendChild(innerDiv);
			
			widget.id = params.id;
			widget.helper = params.helper;
			
			widget._selectTabHandler = dojo.connect(widget.domNode, "onclick", this, this._selectTabHandler);
			this.inherited(arguments);
			
			this.tabTargets[widget.index] = new Target(widget.domNode,{accept: ["viewlet"], index: widget.index, dashboardContainer: this.dashboardContainer});
		},
		createNewTab: function(event){
			if (event) {
				dojo.stopEvent(event);
			}
			this.dashboardContainer.dashboardPage.setLoading(true, messages.creatingTab);
			// first get a UUID
			var serviceObject = {
				self: this.draggableTabs,
				success: function(uuid) {
					// add the viewlet at top of first column
					var columns = new Array();
					for (var i=0; i<3; i++) {
						columns.push(new ColumnHelper({}, this));
					}
					var rawHelper = {
						id: uuid,
						title: messages.newTab,
						columns: columns
					};
					var viewletPageHelper = new ViewletPageHelper(rawHelper, this.self.dashboardItem);
					
					this.self.dashboardItem.addViewletPage(viewletPageHelper);
					var index = this.self.widgets.length;
					this.self.add(null,{title: viewletPageHelper.getTitle(), id: viewletPageHelper.getId(), index: index, helper: viewletPageHelper, type: "tab"});
					this.self._selectTabAction(this.self.widgets[index]);
					this.self.dashboardContainer.dashboardPage.setLoading(false);
				},
				failure: function(errorObject) {
					this.self.dashboardContainer.dashboardPage.setLoading(false);
					console.error("ViewletPage#addViewlet remote error: " + errorObj.message);
				}
			};
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
			DashboardClient.generateUUID(srh);
		},
		destroy: function() {
			for (var i = 0; i < this.widgets.length; i++) {
				dojo.disconnect(this.widgets[i]._selectTabHandler);
			}
			this.inherited(arguments);
			dojo.disconnect(this._menuItemClickHandler);
			dojo.disconnect(this._tabTitleClickStopper);
			dojo.disconnect(this._tabTitleKeypressStopper);
			if (this._editableTabTitle && this._editableTabTitle.destroy)
				this._editableTabTitle.destroy();
			if (this._menu) {
				this._menu.destroy();
				delete this._menu;
				delete this._menuIsEditing;
				delete this._menuDeleteItem;
			}
		},
		move: function(fromIndex, toContainerIndex, toIndex) {
			try {
				//update server
				this.dashboardItem.moveViewletPage(fromIndex, toIndex);
				//update viewletPage node cache
				this.dashboardContainer.moveViewletPage(fromIndex, toIndex);
				//reorganize target array
				var target = this.tabTargets[fromIndex];
				this.tabTargets.splice(fromIndex,1);
				this.tabTargets.splice(toIndex, 0, target);
				this._updateTargetIndices(this.tabTargets);
			} catch (err) {
				console.error("New viewlet position could not be saved: " + err.message + " - Refreshing viewlet page.");
			}
			this.inherited(arguments);
		},
		_updateTargetIndices: function(targets) {
			for (var i = 0; i < targets.length; i++) {
				if (targets[i])	targets[i].index = i;
			}
		},
		onDndCancel: function(success) {
			if(draggableManager().widget != this.selectedTab) {
				dojo.removeClass(draggableManager().widget.domNode,"selectedTab");	
			}
			this.inherited(arguments);
		},
		setEditMode: function(editing) {
			this.isEditing = editing;
			if (this.dashboardItem.getPermissions().addTab) {
				this._disabled = !editing;
			} else {
				this._disabled = true;
			}
			if (editing) {
				dojo.removeClass(this.node,"dndContainerDisabled");
				if (this.selectedTab){
					this._addEditableTabTitle();
					this._addTabDropDown();	
				}
			} else {
				dojo.addClass(this.node,"dndContainerDisabled");
				if (this.selectedTab) {
					this._removeEditableTabTitle();
					this._addTabDropDown();
				}
			}
		},
		startup: function() {
			for (var i = 0; i < this.widgets.length; i++) {
				if (this.widgets[i].id == this.selectedTabId) {
					this.selectTab(i);
					return;
				}	
			}
			if (this.widgets[0]) {
				this.selectTab(0);	
			}
		},
		remove: function(index){
			if(this.widgets.length == 1) {
				alert(messages.deleteLastTab);
				return;
			}
			var widget = this.widgets[index];
			if (!widget) return;
			
			if (confirm(dojo.string.substitute(messages.deletingTabConfirm, [ widget.helper._title ]))) {
				this.dashboardContainer.dashboardPage.setLoading(true, messages.deletingTab);
				this.inherited(arguments); 
				this.dashboardItem.deleteTab(index);
				this.dashboardContainer.deleteCachedViewletPage(index);
				widget.domNode.parentNode.removeChild(widget.domNode);
				this.selectTab(0);
				var perms = this.dashboardItem.getPermissions();
				if (perms.addTab) {
					if (this.widgets.length < 6) this.dashboardContainer.metaNav._createNewTabLink();
				}
				this.dashboardContainer.dashboardPage.setLoading(false);
			} else {
				return;
			}
		},
		selectTab: function(index) {
			if (this.widgets[0] && (!index || index > this.widgets.length)) {
				var tab = this.widgets[0];
			} else {
				var tab = this.widgets[index];	
			}
			if (this.selectedTab) {
				dojo.removeClass(this.selectedTab.domNode,"selectedTab");
			}
			dojo.addClass(tab.domNode, "selectedTab");
			if (this.selectedTab) {
				this.tabTargets[this.selectedTab.index] = new Target(this.selectedTab.domNode,{accept: ["viewlet"], index:this.selectedTab.index, dashboardContainer: this.dashboardContainer});
				if (this.tabTargets[index]) {
					this.tabTargets[index].destroy();
					this.tabTargets[index] = null;
				}
				this._removeTabDropDown();
				this._removeEditableTabTitle();
			}
			this.dashboardContainer.showViewletPage(tab.index, tab.helper);	
			this.selectedTabId = tab.id;
			this.dashboardContainer.selectedTabId = tab.id;
			this.selectedTab = tab;
			if (this.tabTargets[index]) {
				this.tabTargets[index].destroy();
				this.tabTargets[index] = null;	
			}
			this._addEditableTabTitle();
			this._addTabDropDown();
		},
		_addEditableTabTitle: function() {
			if (this.isEditing) {
				//if editing, create editable tab title
				var div = document.createElement("div");
				div.className = "editable-tab-title";
				var titleArea = this.selectedTab.domNode.firstChild.firstChild;
				titleArea.innerHTML = "";
				this._editableTabTitle = this.dashboardContainer.createTitle(this.selectedTab.helper._title,div,this._onTabTitleChange, this, "50", "200px");
				titleArea.appendChild(div);
				this._tabTitleKeypressStopper = dojo.connect(this._editableTabTitle.domNode.parentNode,"onkeypress",this, this._stopBubble);
				this._tabTitleClickStopper = dojo.connect(this._editableTabTitle.domNode.parentNode,"onmousedown", this, this._stopBubble);
			}
		},
		_addTabDropDown: function() {
			if (this._menuIcon) {
				this._removeTabDropDown();
			}
			dojo.addClass(this.selectedTab.domNode, "tabWithMenu");
			var menuDiv = document.createElement("div");
			menuDiv.className = "menu";
			var menuLink = document.createElement("a");
			menuLink.href="#";
			menuLink.id = "tabDropDownIcon";
			menuDiv.appendChild(menuLink);
			this.selectedTab.domNode.firstChild.firstChild.appendChild(menuDiv);
			this._menuIcon = menuDiv;
			this._menuItemClickHandler = dojo.connect(menuLink, 'onclick', dojo.hitch(this, this._openTabDropDown));
		},
		_changeColumns: function(event) {
			this.self.dashboardItem.changeColumns(this.self.selectedTab.index,this.toCols);
			this.self._refreshCurrentTab();
			
			var iconClass = this.self._getColumnIconClass(this.self._currentTab);
	
			this.self._popupMenu.domNode.firstChild.firstChild.className = "dijitMenuItemIcon " + iconClass;
		},
		_closeTabDropDown: function(event) {
			dojo.disconnect(this._tabMenuCloseHandle);
			dojo.removeClass(this._menuIcon.firstChild,Constants.ViewletContainer.OPEN_MENU_CLASS);
			dijit.popup.close(this._menu);
		},
		_createTabDropDown: function (){
			if (!this._menu || this._menuIsEditing != this.isEditing) {
				// create items
				var menu = new dijit.Menu();
				dojo.addClass(menu.domNode, "com-ibm-team-dashboard-content-tabmenu");
				dojo.addClass(menu.domNode, "com-ibm-team-dashboard-metanav-tabdropdown");
				if (this.isEditing) {
					menu.addChild(new dijit.MenuItem({label: messages.addViewletText, iconClass: 'addIcon', onClick: dojo.hitch(this, this._expandChooser)}));
					menu.addChild(new dijit.MenuSeparator());
					menu.addChild(new dijit.MenuItem({label: messages.rename, iconClass: 'renameIcon',	onClick: dojo.hitch(this, this._renameTab)}));
				}
				menu.addChild(new dijit.MenuItem({label: messages.refresh, iconClass: 'refreshIcon', onClick: dojo.hitch(this.parent, this.parent._cancelAreYouSure)}));
				if (this.isEditing) {
					this._menuDeleteItem = new dijit.MenuItem({label: messages.deleteText, iconClass: 'deleteIcon', onClick: dojo.hitch(this, this._deleteTab)});
					menu.addChild(this._menuDeleteItem);
					menu.addChild(new dijit.MenuSeparator());
					var menu2 = new dijit.Menu();
					menu2.addChild(new dijit.MenuItem({label: messages.oneColumn, iconClass: 'oneColumnIcon', onClick: this._changeColumns, toCols: 1, self: this}));
					menu2.addChild(new dijit.MenuItem({label: messages.twoColumns, iconClass: 'twoColumnsIcon', onClick: this._changeColumns, toCols: 2, self: this}));
					menu2.addChild(new dijit.MenuItem({label: messages.threeColumns, iconClass: 'threeColumnsIcon', onClick: this._changeColumns, toCols: 3, self: this}));
					var iconClass = this._getColumnIconClass();
					var popupMenu = new dijit.PopupMenuItem({label: messages.layout, iconClass: iconClass});
					popupMenu.popup = menu2;
					menu.addChild(popupMenu);
				}
				menu.startup();
				if (this.isEditing) {
					popupMenu.startup();
					menu2.startup();
					this._popupMenu = popupMenu;
				}
				this._menu = menu;
				this._menuIsEditing = this.isEditing;
			}
			// enable/disable items appropriately
			if (this.isEditing) {
				this._menuDeleteItem.domNode.style.display = (this.widgets.length > 1) ? "" : "none";
			}
		},
		_deleteTab: function(event) {
			this.remove(this.selectedTab.index);
		},
		_expandChooser: function() {
			var currentPage = this.dashboardContainer.getCurrentPage();
			if (currentPage) {
				currentPage.expandViewletChooser();
			}
		},
		_getColumnIconClass: function(){
			var cols = this.dashboardContainer.getCurrentPage().numColumns;
			
			var iconClass;
			if (cols == 1) {
				iconClass = 'oneColumnIcon';
			} else if (cols == 2){
				iconClass = 'twoColumnsIcon';
			} else {
				iconClass = 'threeColumnsIcon';
			}
			
			return iconClass;
		},
		_onTabTitleChange: function(/* string */ newTitle) {
			if(!newTitle) return false;
			this.dashboardItem.setTabTitle(newTitle, this.selectedTab.index);
			this.dashboardContainer.dashboardPage.updateBrowserTitle();
			
			//workaround to innerHTML bug where extra <a> tag wrapper appears when it shouldn't.
			this._editableTabTitle.displayNode.innerHTML = "";
			this._editableTabTitle.displayNode.appendChild(document.createTextNode(newTitle));
		},
		_openTabDropDown: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (this.dashboardContainer.currentPage.killPopup){
				this.dashboardContainer.currentPage.killPopup();
			}
			//calculate the position of the drop down menu
			var pos = dojo.coords(this._menuIcon, true);
			
			//Display menu
			this._createTabDropDown();
			dijit.popup.open({ popup: this._menu, parent: this, x: pos.x, y: pos.y + 2 + Constants.ViewletContainer.LAST_COL_MENU_OFFSET, onExecute: dojo.hitch(this,this._closeTabDropDown),orient: 'TL' });
	
			//register event handlers that cause menu to close
			this._tabMenuCloseHandle = dojo.connect(document.body,"onclick",this,this._closeTabDropDown);
			
			//style menu icon to indicate open state
			dojo.addClass(this._menuIcon.firstChild,Constants.ViewletContainer.OPEN_MENU_CLASS);
			this.dashboardContainer.currentPage.killPopup = dojo.hitch(this, this._closeTabDropDown);
		},
		_refreshCurrentTab: function() {
			if (this.dashboardContainer.getCurrentPage()._addViewletAreaExpanded) {
				var expandAddViewletArea = true;
			}
			this.dashboardContainer.setDirty(this.selectedTab.index);
			this.selectTab(this.selectedTab.index);
			
			var currentPage = this.dashboardContainer.getCurrentPage();
			if (expandAddViewletArea) {
				currentPage.expandViewletChooser(null,true);
			}
		},
		_removeEditableTabTitle: function() {
			if (this._editableTabTitle){
				//if an editableTabTitle already exists, destroy it
				dojo.disconnect(this._tabTitleClickStopper);
				dojo.disconnect(this._tabTitleKeypressStopper);
				this._editableTabTitle.destroy();
				this._editableTabTitle = null;
				this.selectedTab.domNode.firstChild.firstChild.innerHTML = HTMLUtility.escapeHTML(this.selectedTab.helper._title);
			}
		},
		_removeTabDropDown: function() {
			if (this._menuIcon) {
				//if a menuicon currently exists, destroy it
				dojo.disconnect(this._menuItemClickHandler);
				dojo.removeClass(this.selectedTab.domNode, "tabWithMenu");
				if (this._menuIcon.parentNode){
					this._menuIcon.parentNode.removeChild(this._menuIcon);
				}	
				this._menuIcon = null;
			}
		},
		_renameTab: function(event) {
			this._editableTabTitle._onClick(event);
		},
		_selectTabAction: function(/*Widget*/ widget) {
			this.dashboardContainer.dashboardPage.switchingTabs = true;
			location.href = Util.getDashboardItemHref(this.dashboardItem, widget.id);
		},
		_selectTabHandler: function(event) {
			event.preventDefault();
			if (dojo.hasClass(event.currentTarget,"selectedTab")) return;
			
			//if the current tab is being edited, save it
			if (this._editableTabTitle && this._editableTabTitle.editing){
				this._editableTabTitle.save();
			}
			
			//kill any open popup menus
			if (this.selectedTab) {
				try {
					var vp = this.dashboardContainer.viewletPages[this.selectedTab.index];
					if (vp && vp.killPopup) vp.killPopup();
				} catch (e) {
					//eat the error
				}	
			}
			
			for (var i = 0; i < this.widgets.length; i++) {
				if (event.currentTarget == this.widgets[i].domNode) {
					this._selectTabAction(this.widgets[i]);
					break;
				}
			}
		},
		_selectTabById: function(pageId) {
			if (!pageId) {
				pageId = this.dashboardItem.getViewletPages()[0].getId();
			}
			var children = this.widgets;
			for (var x = 0; x < children.length; x++) {
				if (children[x].id == pageId) {
					this.selectTab(x);
					break;
				}
			}
		},
		_stopBubble: function(event){
			if (event) {
				if (this._editableTabTitle.editing) {
					event.cancelBubble = true;	
				} else {
					return;
				}
			}
			return false;
		}
	});
})();
