/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.settings.PreferenceTab");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.ObfUtil");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {
	
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var ObfUtil = com.ibm.team.dashboard.web.util.internal.ObfUtil;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.PreferenceTab",
	com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab, {

	parseOnLoad: true,

	constructor: function(args) {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.title = this._messages.settingsPrefs;
	},
	
	getNumberOfFields: function() {
		return this._settingsArea.viewletDefinition.preferenceDefinitions.length;
	},
	
	isRequired: function() {
		if (this._settingsArea.viewletDefinition.preferenceDefinitions &&
				this._settingsArea.viewletDefinition.preferenceDefinitions.length > 0)
			return true;
		return false;
	},
	
	settingChanged: function(settingId) {
		var prefsToUpdate = this._settingsArea.viewlet.settingChanging(settingId);
		if (prefsToUpdate && prefsToUpdate.length) {
			for (var i = 0; i < prefsToUpdate.length; i++) {
				for (var j = 0; j < this._computedCombos.length; j++) {
					if (this._computedCombos[j] && prefsToUpdate[i] == this._computedCombos[j].select.name) {
						var value = this._computedCombos[j].select.options[this._computedCombos[j].select.selectedIndex].value;
						this._computedCombos[j].field.innerHTML = "";
						this.createFormField(this._computedCombos[j].data, this._computedCombos[j].field, value);
						this._computedCombos.splice(j,1);
						break;
					}
				}
			}
		}
	},
	
	initWorkingCopy: function() {
		var preferences = new Object();
		var prefDefs = this._settingsArea.viewletDefinition.preferenceDefinitions;
		var viewlet = this._settingsArea.viewlet;
		for (var i = 0; i < prefDefs.length; i++) {
			preferences[prefDefs[i].id] = this._settingsArea.viewlet.getPreference(prefDefs[i].id);
		}
		this._settingsArea.workingCopy.preferences = preferences;
	},
	
	getDefaultValue: function(data) {
		return this._settingsArea.workingCopy.preferences[data.id];
	},
	
	setup: function() {
		this._customPreferences = new Object();
		this._settingsArea.tabContainer.addChild(this);
		this.startup();
		this.domNode.appendChild(this.createFormElements(this._settingsArea.viewletDefinition.preferenceDefinitions));
	},
	
	saveSettings: function(newSettings) {
		this._settingsArea.viewletHelper.setPreferences(this._settingsArea.workingCopy.preferences);
	},
	
	_updateCustomPreferenceLabel: function(prefId, value) {
		if (this._customPreferences[prefId] && this._customPreferences[prefId].timeout)
			clearTimeout(this._customPreferences[prefId].timeout);
		var labelHTML = this._settingsArea.viewlet.getPreferenceProvider(prefId).getLabel(value);
		if (!labelHTML) {
			labelHTML = this._settingsArea.viewlet.getPreferenceProvider(prefId).getLoadingLabel();
			this._customPreferences[prefId].timeout = setTimeout(dojo.hitch(this, function() {
				this._updateCustomPreferenceLabel(prefId, value);
				}), 250);
		}
		this._customPreferences[prefId].label.innerHTML = labelHTML;
	},
	
	_showCustomPreferenceChooser: function(event){
		dojo.stopEvent(event);
		
		this._settingsArea.viewlet.getPreferenceProvider(event.target.name).createChooser(
			dojo.hitch(this,this._handleCustomChange), 
			this._settingsArea.workingCopy,
			this._settingsArea.viewlet);
	},
	
	createFormField: function(data, field, value){
		switch(data.preferenceType) {
		case Constants.SettingsArea.FIELD_INTEGER:
			// show empty string if not specified
			var stringValue = "";
			if (value) {
				stringValue += value;
			}
			var inputDiv = document.createElement("div");
			var textBox = new dijit.form.TextBox({name: data.id, value: stringValue, trim: true});
			textBox.startup();
			textBox.domNode.className += " integer";
			if (data.startDisabled)
				textBox.domNode.disabled = true;
			this.connect(textBox.domNode, "onchange", this._handleTextboxChange);
			// workaround for dojo bug: onchange not triggering on blur
			this.connect(textBox.domNode, "onblur", this._handleTextboxChange);
			inputDiv.appendChild(textBox.domNode);
			field.appendChild(inputDiv);
			return textBox.domNode;
		case Constants.SettingsArea.FIELD_PASSWORD:
			var inputDiv = document.createElement("div");
			var textBox = new dijit.form.TextBox({type: "password",name: data.id, value: value, trim: true});
			textBox.startup();
			textBox.domNode.className += " string";
			if (data.startDisabled)
				textBox.domNode.disabled = true;
			this.connect(textBox.domNode, "onchange", this._handlePasswordChange);
			// workaround for dojo bug: onchange not triggering on blur
			this.connect(textBox.domNode, "onblur", this._handlePasswordChange);
			inputDiv.appendChild(textBox.domNode);
			field.appendChild(inputDiv);
			return textBox.domNode;
		case Constants.SettingsArea.FIELD_CUSTOM:
			var inputDiv = document.createElement("div");
			dojo.addClass(inputDiv, "custom-container");

			// Label
			var label = document.createElement("span");
			label.className = "custom-label";
			if (!(data.id in this._customPreferences))
				this._customPreferences[data.id] = new Object();
			this._customPreferences[data.id].label = label;
			this._updateCustomPreferenceLabel(data.id, value);
			
			// Button
			var button = document.createElement("button");
			button.name = data.id;
			this.connect(button,"onclick", this._showCustomPreferenceChooser);
			button.innerHTML = this._messages.settingsPrefsEdit;
			if (data.startDisabled)
				button.disabled = true;
			var buttonSpan = document.createElement("span");
			buttonSpan.className = "custom-edit-button";
			buttonSpan.appendChild(button);

			inputDiv.appendChild(buttonSpan);
			inputDiv.appendChild(label);
			field.appendChild(inputDiv);
			return;
		}
		return this.inherited(arguments);
	},
	_handleCheckboxChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.preferences[box.name] = box.checked;
		this._settingsArea.settingChanged(box.name);
	},
	_handleTextboxChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.preferences[box.name] = box.value;
		this._settingsArea.settingChanged(box.name);
	},
	_handlePasswordChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.preferences[box.name] = ObfUtil.menc(box.value);
		this._settingsArea.settingChanged(box.name);
	},
	_handleComboboxChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.preferences[box.name] = box.options[box.selectedIndex].value;
		this._settingsArea.settingChanged(box.name);
	},
	_handleCustomChange: function(value, prefId) {
		this._settingsArea.workingCopy.preferences[prefId] = value;
		this._updateCustomPreferenceLabel(prefId, value);
		this._settingsArea.settingChanged(prefId);
	}
		
});

})();
