/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectArea");

/*
 * Imports 
 */
dojo.require("dojo.i18n");  
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ListBox");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MultiSelectMessages");

/*
 * Aliases 
 */
var ListBox = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ListBox;

/*
 * TODOs 
 */


(function() {

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectArea",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Properties
	 */	
	context: null,
	unselectedListTitleText: "",
	selectedListTitleText: "",

	/*
	 * Specify the entry field to use when sorting list box entries
	 * Options: "label", "value", "id", "type"
	 */		
	sortField: "label",	

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/utils/MultiSelectArea.html"),
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return MultiSelectArea
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MultiSelectMessages");
		this.unselectedListTitleText = this.messages.msgAllItems;
		this.selectedListTitleText = this.messages.msgSelectedItems;
		this.msgAdd = this.messages.msgAdd;
		this.msgAddAll = this.messages.msgAddAll;
		this.msgRemove = this.messages.msgRemove;
		this.msgRemoveAll = this.messages.msgRemoveAll;
		this.msgOutOf = this.messages.msgOutOf;
		this.msgOK = this.messages.msgOK;
		this.msgCancel = this.messages.msgCancel;
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited(arguments);
		
		// Static Variables //------------------------------------------------->
		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		this._selectedList = null;
		this._unselectedList = null;

		this.configureUI();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectArea";

	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	configureUI: function() {
		// Unselected list title
		this._unselectedListTitle.innerHTML = this.unselectedListTitleText; 
		// Selected list title
		this._selectedListTitle.innerHTML = this.selectedListTitleText;
		
		var self = this;
		
		// Unselected list box
		this._unselectedList = new ListBox({
			height: "250px",
			width: "250px",
			margin: "3px 10px 10px 20px",
			sortField: this.sortField,
			callbackDoubleClick: function(entry) {
				self._handleEventDoubleClickedUnselected(entry);
			}
		});
		this._unselectedListWidget.appendChild(this._unselectedList.domNode);
		
		// Selected list box		
		this._selectedList = new ListBox({
			height: "250px",
			width: "250px",
			margin: "3px 10px 10px 10px",
			sortField: this.sortField,
			callbackDoubleClick: function(entry) {
				self._handleEventDoubleClickedSelected(entry);
			}
		});
		this._selectedListWidget.appendChild(this._selectedList.domNode);

		// Add button
		this.connect(this._addButton, "onclick", this._handleEventAddButtonClicked);
		// Add all button
		this.connect(this._addAllButton, "onclick", this._handleEventAddAllButtonClicked);
		// Remove button
		this.connect(this._removeButton, "onclick", this._handleEventRemoveButtonClicked);
		// Remove all button
		this.connect(this._removeAllButton, "onclick", this._handleEventRemoveAllButtonClicked);

		// OK button
		this.connect(this._okButton, "onclick", this._handleEventOKButtonClicked);
		// Cancel button
		this.connect(this._cancelButton, "onclick", this._handleEventCancelButtonClicked);

		// Key up listener
		this._keyUpHandler = this.connect(dojo.doc, "onkeyup", this._handleEventKeyUp);

	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	_handleEventDoubleClickedUnselected: function(entry) {
		// Add entry to selected list
		this._selectedList.addEntry(entry);
		// Remove entry from unselected list
		this._unselectedList.removeEntry(entry);

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	_handleEventDoubleClickedSelected: function(entry) {
		// Add entry to unselected list
		this._unselectedList.addEntry(entry);
		// Remove entry from selected list
		this._selectedList.removeEntry(entry);

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventAddButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get selected entries
		var entries = this._unselectedList.getSelectedEntries();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to selected list
		this._selectedList.addEntries(entries);
		// Remove entries from unselected list
		this._unselectedList.removeEntries(entries);

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventAddAllButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get all entries
		var entries = this._unselectedList.getAllEntries();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to selected list
		this._selectedList.addEntries(entries);
		// Remove entries from unselected list
		this._unselectedList.removeAllEntries();

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventRemoveButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get selected entries
		var entries = this._selectedList.getSelectedEntries();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to unselected list
		this._unselectedList.addEntries(entries);
		// Remove entries from selected list
		this._selectedList.removeEntries(entries);

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventRemoveAllButtonClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get all entries
		var entries = this._selectedList.getAllEntries();
		// Validate entries
		if (entries.length === 0) {
			// NO-OP
			return;
		}
		// Add entries to unselected list
		this._unselectedList.addEntries(entries);
		// Remove entries from selected list
		this._selectedList.removeAllEntries();

		this._updateCount();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventOKButtonClicked: function(event) {
		dojo.stopEvent(event);
		// Redirect
		this.context._handleEventOKButtonClicked();
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventCancelButtonClicked: function(event) {
		dojo.stopEvent(event);
		// Redirect
		this.context._handleEventCancelButtonClicked();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventKeyUp: function(event) {
		dojo.stopEvent(event);
		
		if (event.keyCode === dojo.keys.ENTER) {
			// Handle enter key presses
			// Default action is clicking OK button 
			// Redirect
			this.context._handleEventOKButtonClicked();
		} else if (event.keyCode === dojo.keys.ESCAPE) {
			// Handle escape key presses
			// Default action is clicking Cancel button 
			// Redirect
			this.context._handleEventCancelButtonClicked();
		}
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	setSelectedEntries: function(entries) {
		this._selectedList.removeAllEntries();
		this._selectedList.addEntries(entries);
		this._updateCount();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getSelectedEntries: function() {
		return this._selectedList.getAllEntries();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	setUnselectedEntries: function(entries) {
		this._unselectedList.removeAllEntries();
		this._unselectedList.addEntries(entries);
		this._updateCount();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getUnselectedEntries: function() {
		return this._unselectedList.getAllEntries();
	},

	/*
 	 * Public Method
 	 * 
 	 * Type is the key (Number)
 	 * Icon path is the value (String)
 	 * 
 	 * @param Array map : [Number]=String
 	 * @return void
 	 */	
	setIconMap: function(map) {
		this._unselectedList.setIconMap(map);
		this._selectedList.setIconMap(map);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */	
	showLoading: function(show) {
		this._unselectedList.showLoading(show);
		this._selectedList.showLoading(show);
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_updateCount: function() {
		
		var sCount = this._selectedList.getEntryCount();
		var uCount = this._unselectedList.getEntryCount();
		
		this._selectionCount.innerHTML = sCount;
		this._totalCount.innerHTML = sCount + uCount;
		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		// Destroy widgets
		this._selectedList.destroy();
		this._unselectedList.destroy();
		
		// Disconnect handlers
		if (this._keyUpHandler !== null) {
			this.disconnect(this._keyUpHandler);
			this._keyUpHandler = null;
		}
		
		// Call super
		this.inherited(arguments);
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}		
	
});

})();
