/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");

(function() {

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var EditorUtil				= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;

// delay between user input and running a search (in milliseconds)
var DELAY					= 750;

// cap max number of query results at 50
var MAX						= 50;

var KEY_RETURN			= 13;
var KEY_ENTER			= 14;
var KEY_SPACE			= 32;
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemContent",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/SelectWorkItemContent.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/SelectWorkItemContent.css"),
		constructor: function(params) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.labelEnterSearchString = this.messages.labelEnterSearchString;
			this.labelRecentlyViewed = this.messages.labelRecentlyViewed;
			this.buttonOK = this.messages.buttonOK;
			this.buttonCancel = this.messages.buttonCancel;
			this.htmlEnterSearchString = this.messages.labelEnterSearchString;
		},
		postCreate: function() {
			this._connectEvents();
			// mark that keypresses from this control should be ignored w/r/t
			// dirtying the editor
			this.queryInput.__jazzIgnoreKeystroke= true;
		},
		
		setFocus: function() {
			this._showRecentlyViewedList();
			this.queryInput.focus();
		},
		
		reset: function() {
			this.queryInput.value= "";
			this.queryResults.length= 0;
			this._clearStatusMessage();
			this.okCmd.disabled= true;
		},
		
		setKey: function(key) {
			this.key= key;
		},
		
		setAllowsMultiple: function(allowsMultiple) {
			this.allowsMultiple= allowsMultiple;
		},
		
		setExclusionList: function(exclusionList) {
			this.exclusionMap= {};
			for (var i=0; i<exclusionList.length; i++) {
				var id= exclusionList[i];
				this.exclusionMap[id]= true;
			}
		},
		
		_connectEvents: function() {
			this.connect(this.okCmd, "onclick", "_handleOk");
			this.connect(this.queryResults, "ondblclick",  "_handleOk");
			this.connect(this.cancelCmd, "onclick",  "_handleCancel");
			this.connect(this.queryInput, "onkeyup", "_doCancellableSearch");
			this.connect(this.queryResults, "onkeyup",  "_handleQueryResultsKey");
			this.connect(this.queryResults, "onchange", "_handleSelectionChange");
		},
		
		_handleQueryResultsKey: function(e) {
			if(e.keyCode === KEY_ENTER || e.keyCode === KEY_RETURN || e.keyCode === KEY_SPACE) {// enter
				this._handleOk();
			}
		},
		
		_handleOk: function() {
			var retArr= [];
			var options= this.queryResults.options;
			for (var i=0; i<options.length; i++) {
				if (options[i].selected) {
					retArr.push(this._backingWorkItems[options[i].value]);
				}
			}
			this.onOk(retArr, this.key);
			this.reset();			
		},
		
		_handleCancel: function() {
			this.onCancel();
			this.reset();
		},
		
		_handleSelectionChange: function() {
			// disable the 'OK' button if nothing is selected
			this.okCmd.disabled= this.queryResults.selectedIndex === -1;
			if (!this.allowsMultiple) {
				this._deselectPreviouslySelected();
			}
		},
		
		_deselectPreviouslySelected: function() {
			var options= this.queryResults.options;
			var prevOption= this._currentOption ? this._currentOption : null;
			var newOptionSet= false;

			for (var i=0; i<options.length; i++) {
				if (!options[i].selected) {
					continue;
				} else if (options[i] == prevOption || newOptionSet) {
					options[i].selected= false;
				} else {
					this._currentOption= options[i];
					newOptionSet= true;
				}
			}
		},
		
		_doCancellableSearch: function(e) {
			if(e.keyCode === 40){ // arrow down 
				this.queryResults.focus();
				return;
			} 
			if (this._currentId) {
				clearTimeout(this._currentId);
			}
			var self= this;
			var doSearchFunc= function() {
				self._doSearch();
			};
			this._currentId= setTimeout(doSearchFunc, DELAY);
		},
		
		_doSearch: function() {
			// if there's no input, do nothing
			var value= this.queryInput.value;
			if (dojo.trim(value) === "") {
				this.queryResults.length= 0;
				this._clearStatusMessage();
				return;
			}
			var srh= new ServiceResponseHandler(this, "_loadQueryResults", "_queryError");
			var params;
			if (WorkItemUtil.isWorkItemId(value))
				params= { id: WorkItemUtil.getWorkItemId(value) };
			else
				params= { fullText: value };
			this._setStatusMessage(this.messages.busyRunningSearch);
			QueryClient.getResults(params, srh);
		},
		/*
		 * Get the recent items that are workitems
		 *    id 
		 *    summary
		 */
		_getRecentWorkItems: function() {
			if(this.parentController && this.parentController.pageController)
					return this.parentController.pageController.getRecentWorkItemList();
			return [];
		},
		/*
		 * Return the current work item id, using lazy initialization
		 */
		_getCurrentWorkItemId: function() {
			if(this._currentWorkItemId) return this._currentWorkItemId;
			var currentWorkItem;
			if(this.parentController && this.parentController.pageController) {
				currentWorkItem = this.parentController.pageController.getCurrentWorkItem();
			} else {
				if(this.parentController.workItem) {
					currentWorkItem = this.parentController.workItem;
				} else {
					if(this.parentController._workItem) {
						currentWorkItem = this.parentController._workItem;
					}
				}
			}
			
			var currentWorkItemId = -1;
			if(currentWorkItem) {
				currentWorkItemId = currentWorkItem.id;
			}
			this._currentWorkItemId = currentWorkItemId;
			return this._currentWorkItemId;
		},
		/*
		 * Show the recently viewed list
		 */
		_showRecentlyViewedList: function() {
			var currentWorkItemId = this._getCurrentWorkItemId();
			try{ 
				var currentWorkItem = this.parentController.pageController.getCurrentWorkItem(); // fix minor bug
				var currentLabel = this.messages.fragmentThisWorkItem;
				if(currentWorkItemId > 0 && currentWorkItem && currentWorkItem.attributes && currentWorkItem.attributes.workItemType && currentWorkItem.attributes.workItemType.label) {
					currentLabel = currentWorkItem.attributes.workItemType.label + " " + currentWorkItemId; 
				}
				var title = dojo.string.substitute(this.messages.labelRecentlyViewedExcluding , [currentLabel]);
				dojox.data.dom.replaceChildren(this.resultTitle, document.createTextNode(title));
			} catch(ignore) {
				this.resultTitle.innerHTML = this.messages.labelRecentlyViewed; /* innerHTML OK */
			};
			this._backingWorkItems= {};
			var recentItems = this._getRecentWorkItems();
			for (var i=0; i<recentItems.length; i++) {
				if(currentWorkItemId == recentItems[i].id || recentItems[i].id < 0) {
					continue; // don't put the current item on the list; don't put new items on the list
				}
				var title= EditorUtil.getSummaryLabel(recentItems[i]);
				var option= new Option(title, recentItems[i].id);
				EditorUtil._addOption(this.queryResults, option, /*isDefault*/false);
				this._backingWorkItems[recentItems[i].id]= recentItems[i];
				if (this.exclusionMap && this.exclusionMap[recentItems[i].id]) {
					this._disableOption(option,recentItems[i].id );
				}
			}
		},
		_loadQueryResults: function(summaryPage) {
			this.resultTitle.innerHTML = this.messages.labelMatching; /* innerHTML OK */
			var summaries= summaryPage.workItemSummaryDTOs ?
					summaryPage.workItemSummaryDTOs : [];
			this.queryResults.length= 0;
			this._backingWorkItems= {};
			var displayedResults= summaries.length;
			for (var i=0; i<summaries.length; i++) {
				var title= summaries[i].id + ": " + EditorUtil.restoreTags(summaries[i].summary);
				var option= new Option(title, summaries[i].id);
				EditorUtil._addOption(this.queryResults, option, /*isDefault*/false);
				this._backingWorkItems[summaries[i].id]= summaries[i];
				if (this.exclusionMap[summaries[i].id]) {
					this._disableOption(option,summaries[i].id);
				}	
			}
			this._setStatusMessage(dojo.string.substitute(this.messages.msgResults, [ displayedResults ]));
		},
		_disableOption: function(option, id){
			var currentWorkItemId = this._getCurrentWorkItemId();
			option.style.color = "#CCCCCC";
			option.selected = false;
			if(id == currentWorkItemId) {
				option.title = this.messages.titleThisIsCurrent;
			} else {
				option.title = this.messages.titleThisIsLinked;
			}
			
		},
		_queryError: function(errorObj) {
			this._setStatusMessage(this.messages.errorFetchingSearchResults, /*isError*/ true);
		},
		
		_clearStatusMessage: function() {
			this.msgArea.innerHTML= ""; /* innerHTML OK */
		},
		
		_setStatusMessage: function(msg, isError) {
			dojox.data.dom.replaceChildren(this.msgArea, document.createTextNode(msg));
			this.msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
		}
	}
);

})();