/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.HTMLWidget");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.HTMLWidget",
	[dijit._Widget, dijit._Templated],
	{

		templateString: '<div class="HtmlWidget" >' +
						'<span style="display:block" dojoAttachPoint="descriptionSPAN" dojoAttachEvent="onchange: _handleChange" ></span>' +
						'<span style="display:block" dojoAttachPoint="descriptionTextSPAN" dojoAttachEvent="onchange: _handleChange" class="DescriptionSpan"></span>' +
						'</div>',
						
		_isChanged: false,
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			var value =  WidgetUtil.getFirstValue(this.values, /* isString*/ true, /* returnEmpty */ true, "content");
			
			// Links need to be resolved in the original read only value because the links refer to offsets 
			if(value && this.values[0] && this.values[0].links && this.values[0].links.length > 0 ) {
				this._originalReadOnlyValue = EditorUtil.resolveLinks(value, this.values[0].links); // handle "bug 1234" in content
			} else  {
				this._originalReadOnlyValue = value;
			}
			
			// Now value is the R/W text, which needs some manipulation
			value = value == null ? "" : dojo.trim(value + ""); // Handle nonstring values;
			
			value = value.replace(/<br\/>/gi, "\n")
				
			this._isReadOnly = this.parentController.pageController.isGuest() || (value.length != 0 && value != "\n" );
	
			this.descriptionTEXT = new com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget({parentController: this.parentController});
			this.descriptionTextSPAN.appendChild(this.descriptionTEXT.domNode);
			
			this._descriptionSpanClassName = "DescriptionSpan";
			this._descriptionSpanEmptyClassName = "DescriptionSpanEmpty";
			if(this._modifiers.isSpanningColumns) {
				this._descriptionSpanClassName = "DescriptionSpan_TwoColumn";
				this._descriptionSpanEmptyClassName = "DescriptionSpanEmpty_TwoColumn";
				this.descriptionSPAN.className = "DescriptionSpan_TwoColumn";
				this.domNode.className = "HtmlWidget_TwoColumn"
				this.descriptionTextSPAN.className = "DescriptionTextSpan_TwoColumn";
				this.descriptionSPAN.className = "DescriptionSpan_TwoColumn";
			}
			this.setSectionMenu = this.creationSpec.setSectionMenu == true;
			this.isDescription = this.creationSpec.isDescription == true;
						 
			if( !this.parentController.pageController.isGuest() && !this._modifiers.isString && (this._modifiers.isSpanningColumns || (this.isDescription && !this.setSectionMenu))) {
				    this._editAction = this.sectionLayout.getAction(this.messages.buttonEdit,  /* icon */  null, /* disable icon */ null, /* icon title */ this.messages.titleEdit,  this, "_toggleHTMLEditor", /* enable */ true);
					dojo.addClass(this._editAction, "SectionActionEmbedded");
					this.descriptionTextSPAN.appendChild(this._editAction);
			}
			
			this.setTextValue(value);

			if(this.setSectionMenu) {
				dojo.addClass(this.descriptionTEXT.textarea, "SelfSizingTextAreaTall");
			} else {
				dojo.addClass(this.descriptionTEXT.textarea, "SelfSizingTextAreaShort");
			}
			if (!this.parentController.pageController.isGuest()) {
				if (this.setSectionMenu && !this._editAction) {
					this._editAction = this.sectionLayout.addAction(this.messages.buttonEdit, /* icon */ null, /* disable icon */ null, /* icon title */ this.messages.titleEdit, this, "_toggleHTMLEditor", /* enable */ true);
				}
				else {
					if (!this._editAction) {
						this._isReadOnly = false;
					}
				}
			}
			
			this._setHTMLEditorState(this._isReadOnly);
			// Watch  for changes
			this._stopChangeInterval();
			this.changeInterval = setInterval(dojo.hitch(this, this._checkForChanges), 600);
				
		},
		destroyWidget: function() {
			this._stopChangeInterval();
			this.destroy();
		},
		getValue: function() {
			return this.getTextValue();
		},
		getTextValue: function() {
			return this.descriptionTEXT.getValue();
		},
		_checkForChanges: function() {
			// IE adds returns to the values so strip them for compare
			if (this._isValueChanged()) {
				this._stopChangeInterval();
				if(this.parentWidget){
					var e = {
						srcElement: this.descriptionTEXT
					};
					this._handleChange(e);
				}
			}
		},
		_isValueChanged: function() {
			var oldValue = this._originalModifiedTextValue.replace(/\r/g,"");
			var newValue = this.getTextValue().replace(/\r/g,"");
			return oldValue != newValue
		},
		_stopChangeInterval: function() {
			if(this.changeInterval) {
				window.clearInterval(this.changeInterval);	
			}
		},
		getFormElementValues: function(formElements) {
			// only include the field if it's modified to avoid	
			if(!this._isChanged && !this._isValueChanged()) return;
			formElements[this.attributeId] = this.getTextValue();
		},
		
		setTextValue: function(value) {
			this._originalUnmodifiedTextValue = value;
			this._originalModifiedTextValue = this._cleanText(value);
			var text =this._originalModifiedTextValue;
			this.descriptionTEXT.setValue(this._originalModifiedTextValue);
		},
		_cleanText:function(value) {
			return EditorUtil.restoreTags(value);
		},
		setFormElementValues: function(retainedState){
		 	if(retainedState[this.attributeId]) {
				var cleanText = this._cleanText(retainedState[this.attributeId]);
				if(this.getTextValue() != cleanText) {
					this.setTextValue(retainedState[this.attributeId]);
					this._isChanged = true;
					if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
						this.parentWidget[this._modifiers.onValueChanged](this , this.getTextValue());
					}
				}
			}
		 },
		_handleChange: function(e) {
			this._isChanged= true;
			this.parentWidget._handleDirtyingEvent(e);
		},
		_toggleHTMLEditor: function() {
			this._setHTMLEditorState(!this._isReadOnly);
			},
		_showHTMLEditor: function() {
			this._setHTMLEditorState(false);
			},
		_showHTMLPreview: function() {
			this._setHTMLEditorState(true);
			},
						
		_setHTMLEditorState: function(isReadOnly) {
			this._isReadOnly = isReadOnly;
			if(!this._modifiers.isString && isReadOnly) {
				if (this._editAction) {
					dojox.data.dom.replaceChildren(this._editAction._jazzAnchor, document.createTextNode(this.messages.buttonEdit));
				}

				// If the value is changed, use the user's value, else the original value with links added.
				// The _getTextValue will not have phrases resolved to links (e.g., "bug 1234")
				// since that is done on the server
				if(this._isValueChanged()) {
					var value = dojo.trim(this.getTextValue());
				} else {
					var value = this._originalReadOnlyValue || "";	
				}

				// TODO: Parse the value 
				var className = this._descriptionSpanClassName;
				if (value.length == 0) {
					className = this._descriptionSpanEmptyClassName;
					if(this._modifiers.noValueLabel) {
						value = this._modifiers.noValueLabel;
					} else {
						value = "&nbsp;"
					}
				}
				else {
						value = EditorUtil.getSafeTextSpan(value, this);
				}

				dojox.data.dom.replaceChildren(this.descriptionSPAN,value);
				EditorUtil.linkAttachmentPreview(value, this.parentWidget._workItem); // must be in dom
				this.descriptionSPAN.className = className
				this.descriptionSPAN.style.display = "block";
				this.descriptionTEXT.domNode.style.display = "none";
				
//				this.sectionLayout.setActionIcon(this._editDescriptionAction, ImagePool.EDIT, "Edit the description");
			} else {
				if (this._editAction) {
					dojox.data.dom.replaceChildren(this._editAction._jazzAnchor, document.createTextNode(this.messages.buttonPreview));
				}
				
				this.domNode.title = "";
				this.descriptionSPAN.style.display = "none";
				this.descriptionTEXT.domNode.style.display = "";
				this.descriptionTEXT.focus();
//				this.sectionLayout.setActionIcon(this._editDescriptionAction, ImagePool.EDIT_DISABLED, "Show the description");
			}
			
		},
		_handleClick: function(e){
			if(!this._isReadOnly) return;
			// If you click on an anchor, just follow the link
			if(e.target.tagName == "A") return;
			this._setHTMLEditorState(false);
		}
		
	}
);

})();
