/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.ui.internal.HoverPopup");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function(){
	
var STYLE_VALID= "validation-ok"
var STYLE_INVALID= "validation-failed"

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/layout/templates/TextWidget.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		this.presentation= args.presentation;
		this.selection= args.selection;
		
		var converters= new Object();
		converters["integer"]= function(value) {
			return parseInt(value); 
		};
		
		converters["long"]= converters["integer"];
		
		converters["float"]= function(value) {
			return parseFloat(value);
		};
		
		if(args.converter) {
			this._hasCustomConverter = true;
			this._converter = args.converter;
			this._converterParms = args.converterParms;
		} else {
			this._converter= converters[this.presentation.attribute.attributeTypeId];
		}
	},
	
	postCreate: function() {
		dojo.removeClass(this._textInput, STYLE_INVALID);
		dojo.addClass(this._textInput, STYLE_VALID);
		if (this._converter) {
			this._tooltipHtml = this.messages.titleEnterNumber;
			this._toolTip = new net.jazz.ajax.ui.internal.HoverPopup({
				callback: dojo.hitch(this, this.populateTooltip),
				isProgrammatic: true
			}, this._tooltipSpan);
		}
		
		if (this.selection)
			this._setValues(this.selection);
	},
	
	/* String[] */ getValues: function() {
		var selection= new Array();
		var rawValues= this._getRawValues();
		for (var i= 0; i < rawValues.length; i++) {
			var value= this._convert(rawValues[i]);
			if (value)
				selection.push(value);
		}
		if(rawValues.length == 0){
			this._convert("");
		}
		return selection;
	},
	doValidate: function() {
		var inputNode= this._textInput;
		if (this._validateInput()) {
			dojo.disconnect(this._onBlur_handle);
			dojo.removeClass(inputNode, STYLE_INVALID);
			dojo.addClass(inputNode, STYLE_VALID);
			if (this._toolTip)
				this._toolTip.close();
		} else {
			dojo.removeClass(inputNode, STYLE_VALID);
			dojo.addClass(inputNode, STYLE_INVALID);
			if (this._toolTip) {
				if(this._hasCustomConverter) {
					this._tooltipHtml = this._converterParms.toolTip;
					var _self = this;
					
					this.connect(inputNode, "onblur", function(e) {
							_self._validateOnBlur(e);
						})				
				}
				this._toolTip.open(inputNode);
			}

		}
	},
	populateTooltip: function() {
		this._tooltipSpan.innerHTML = this._tooltipHtml;
	},
	_validateOnBlur: function(e) {
		dojo.disconnect(this._onBlur_handle);
		dojo.removeClass(this._textInput, STYLE_INVALID);
		dojo.addClass(this._textInput, STYLE_VALID);
		this._toolTip.close(); 
		this._textInput.value = this._converter(this._textInput.value,  this._converterParms,  /* get valid */ true);

	},
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_convert: function(value) {
		if (value.length > 0 && this._converter) {
			return this._converter(value, this._converterParms);
		} else {
			if(this._hasCustomConverter) {
				return this._converter("", this._converterParms);
			}
		}
		return value;
	},
	
	_validateInput: function() {
		if (this._converter) {
			var values= this._getRawValues();
			for (var i= 0; i < values.length; i++) {
				var converted= this._convert(values[i])
				if(converted && converted.isCustom) {
					if(!converted.isValid) {
						return false;
					}
				} else {
					if (isNaN(converted) || (values[i] != converted))
						return false;			
				}
			}
		}
		return true;	
	},
	
	/* String[] */ _getRawValues: function() {
		var selection= new Array();
		var stringInput= this._textInput.value;
		var parts= stringInput.split(",");
		for (var i= 0; i < parts.length; i++) {
			var trimmed= dojo.trim(parts[i]);
			selection.push(trimmed);
		}
		return selection;
	},
	
	_setValues: function(/* UIItemDTO[] */ selection) {
		var list= new Array();
		for (var i= 0; i < selection.length; i++) {
			list.push(selection[i].id);
		}
		this._textInput.value= list.join(", ");
	}
});
})();
