/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.GroupWidget"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.widgets.WorkLoadBar"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var TableTreeItem		= com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem;
var ImageClient			= com.ibm.team.apt.web.client.internal.ImageClient;
var PlanningUtil		= com.ibm.team.apt.web.client.util;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.GroupWidget", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_groupInfo: null,
	_progressInfo: null,
	_plannedItemsTab: null,
	
	_groupContent: null,

	// ---- api ------------------------------------------------------------------------------------------------------------	

	updateBar: function() {
		dojo.query(".bar", this._groupContent).orphan(); //$NON-NLS-1$
		this._createBar(this._groupContent);
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	postCreate: function() {
		this.inherited(arguments);

		var workItemColumnCount= this._plannedItemsTab.getColumnDefinitions().length;

		var cell= document.createElement("td"); //$NON-NLS-1$
		cell.colSpan= workItemColumnCount;

		this._groupContent= document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(this._groupContent, "content"); //$NON-NLS-1$
		dojo.addClass(this._groupContent, "label"); //$NON-NLS-1$

		
		if (this._groupInfo.photo != null) {
			var photoElement= document.createElement("img"); //$NON-NLS-1$
			dojo.addClass(photoElement, "photo"); //$NON-NLS-1$
			photoElement.src= ImageClient.getImageUrl(this._groupInfo.photo);
			this._groupContent.appendChild(photoElement);
		}

		this._createBar(this._groupContent);

		var groupTextContainer= document.createElement("span"); //$NON-NLS-1$
		dojo.addClass(groupTextContainer, "labelContainer"); //$NON-NLS-1$

		var titleElement= document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(titleElement, "title"); //$NON-NLS-1$
		titleElement.appendChild(document.createTextNode(this._groupInfo.name));
		groupTextContainer.appendChild(titleElement);
		
		var countElement= document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(countElement, "counts"); //$NON-NLS-1$
		countElement.appendChild(document.createTextNode(bind(MSGS.GROUPWIDGET_COUNTS, [this._progressInfo.getCloseCount(), this._progressInfo.getOpenCount()])));
		groupTextContainer.appendChild(countElement);
		
		this._groupContent.appendChild(groupTextContainer);
		cell.appendChild(this._groupContent);

		this._getRowContainer().appendChild(cell);

		this._applyTreeLayout(false);
		
		this.connect(this._getRowContainer(), "onclick", "_doExpandToggleState"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	
	_createBar: function(groupContent) {
		var barMode= this._plannedItemsTab.getBarMode();
		var barWidget= barMode != null ? barMode.createBar(this._groupInfo, this._progressInfo, "100%", "10px") : null; //$NON-NLS-1$ //$NON-NLS-2$

		if (barWidget != null) {
			var barContainer= document.createElement("div"); //$NON-NLS-1$
			dojo.addClass(barContainer, "bar"); //$NON-NLS-1$
			barContainer.appendChild(barWidget.domNode);

			var labelProvider= barWidget.getLabelProvider();

			var hoursContainer= document.createElement("div"); //$NON-NLS-1$
			dojo.addClass(hoursContainer, "numbers"); //$NON-NLS-1$
			dojo.addClass(hoursContainer, "hours"); //$NON-NLS-1$
			hoursContainer.appendChild(labelProvider.getHoursTextNode());	
			barContainer.appendChild(hoursContainer);

			var estimateContainer= document.createElement("div"); //$NON-NLS-1$
			dojo.addClass(estimateContainer, "numbers"); //$NON-NLS-1$		
			dojo.addClass(estimateContainer, "estimate"); //$NON-NLS-1$
			estimateContainer.appendChild(labelProvider.getEstimateTextNode());	
			barContainer.appendChild(estimateContainer);

			groupContent.appendChild(barContainer);
		}
	},

	_doExpandToggleState: function(e) {
		dojo.stopEvent(e);
		this.toggleExpansionState();
	},

	_getTreeLayoutElement: function() {
		var queryResult= dojo.query("td > div", this._getRowContainer()); //$NON-NLS-1$
		return queryResult.length > 0 ? queryResult[0] : null;
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.view.plan.planneditems.GroupWidget.constructorArgs= function(groupDescription, progresInfo, parentItem, plannedItemsTab) {
	var result= {
		_groupInfo: groupDescription,
		_progressInfo: progresInfo,
		_plannedItemsTab: plannedItemsTab
	};

	return dojo.mixin(result, TableTreeItem.constructorArgs(parentItem, "group")); //$NON-NLS-1$
};

})();
