/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.query.SavedQueries");

dojo.require("dijit._Widget");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

var ServiceResponseHandler		= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient 				= com.ibm.team.workitem.web.client.internal.QueryClient;
var QueryEventTopics			= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var WorkItemUtil				= com.ibm.team.workitem.web.client.util;
var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ImagePool					= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;

var QUERY_PARTIAL_VIEW_COUNT = 7;


function deleteQuery(e, itemId, messages) {
	
	e.preventDefault();
	
	var handler= {
		success: function() {
			dojo.publish(QueryEventTopics.BUILDER_POST_SAVE, []);
		},
		error: function(e) {
			alert(e.message); // TODO
		}
	}
	if (confirm(messages.confirmDeleteQuery)) {
		var responseHandler= new ServiceResponseHandler(handler, "success", "error");
		QueryClient.deleteQuery(itemId, responseHandler);
	}
}

function createRow(queryItem, navSection, messages) {
	
	// create execute query link
	var href = WorkItemUtil.getRunSavedQueryURI(queryItem.itemId, null, null, null, null, /* isUnique */ true, /*preserve filter*/ false);
	var queryName = WorkItemUtil.getLeftNavString(queryItem.name);
	var queryLink = navSection.addSectionRow(queryName, href, ImagePool.WORKITEM_QUERY);
	queryLink.id = queryItem.itemId + "-QUERY";
	
	// create an 'Edit' link if we're in the 'My Queries' query section
	if (queryItem.isEditable && queryItem.isEditable !== "false") {
		navSection.addArtifactCommand(queryLink, messages.buttonDelete, function(e) { deleteQuery(e, queryItem.itemId, messages)}, "delete-query-command", true);
		navSection.addArtifactCommand(queryLink, messages.buttonEdit, WorkItemUtil.getEditQueryURI(queryItem.itemId), "edit-query-command", true);
	}

	return queryLink;
}

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.query.SavedQueries",
	dijit._Widget,
	{
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		},
		populate: function(navSection) {
			if (!this.parentController) {
				throw new Error("SavedQueries#postCreate:" + this.messages.errorMissingRequiredParameterParentController);
			}
			if (!this.queryType) {
				throw new Error("SavedQueries#postCreate:" + this.messages.errorMissingRequiredParameterQueryType);
			}
			if (this.queryType !== "shared" && this.queryType !== "myQueries" && this.queryType !== "child") {
				throw new Error("SavedQueries#postCreate:"+ dojo.string.substitute(this.messages.errorInvalidParmValue, [ this.queryType ] )); 
			}
			
			this.navSection = navSection;
			
			// If we're the 'My Queries' section subscribe to the query builder's
			// 'Save' operations by refreshing ourself. 
			if (this.queryType === "myQueries")
				dojo.subscribe(QueryEventTopics.BUILDER_POST_SAVE, this, this._refreshSavedQueries);
			
			if (this.queryType === "shared")
				this._sharedSections = new Object();

			// subscribe to initialize the section
			if (this.queryType !== "child")
				dojo.subscribe(QueryEventTopics.INITIALIZE_QUERIES, this, this._initializeSavedQueries);
		},
		_isInitialized: false,
		_initializeSavedQueries: function() {
			if(!this._isInitialized) {
				this._refreshSavedQueries();
				this._isInitialized = true;
			} else {
				this._uniquifyQueryURLs()
			}
		},
		
		_uniquifyQueryURLs: function() {
			if (this.queryType === "shared" && this._sharedSections)
				for (var i in this._sharedSections)
					this._sharedSections[i]._uniquifyQueryURLs();
			else {
				if (this._queryLinks) {
					for (var i = 0; i < this._queryLinks.length; i++)
						this._queryLinks[i].firstChild.href = WorkItemUtil.reuniquifyURL(this._queryLinks[i].firstChild.href);
				}
			}
		},
		
		_refreshSavedQueries: function() {

			var responseHandler= new ServiceResponseHandler(this, "_loadSavedQueries", "_loadSavedQueriesError");

			if (this.queryType === "myQueries") {
				var args= {ownerItemId: getAuthenticatedContributor().itemId};
				QueryClient.getQueriesForOwner(args, responseHandler);
			} else if (this.queryType === "shared") {
				this._refreshSharedSections();
			}
	
		},
		
		_refreshSharedSections: function() {
			var serviceObject = {
				self: this,
				success: function(result) {
					result.sort(function(a,b) {
						if (a.scope == QueryClient.SCOPE_OTHER_CONTRIBUTORS || b.scope == QueryClient.SCOPE_PROJECT_AREA)
							return 1;
						if (b.scope == QueryClient.SCOPE_OTHER_CONTRIBUTORS || a.scope == QueryClient.SCOPE_PROJECT_AREA)
							return -1;
						return a.scopeName > b.scopeName ? 1 : -1;
					});
					for (var i = 0; i < result.length; i++) {
						if (!this.self._sharedSections[result[i].scopeItemId]) {
							var subWidget = new SavedQueries({parentController: this.self.parentController, queryType: "child"})
							var name = result[i].scope == 1 ? this.self.messages.labelIndividuallyShared : result[i].scopeName;
							switch(result[i].scope) {
								case QueryClient.SCOPE_OTHER_CONTRIBUTORS:
									var name = this.self.messages.labelIndividuallyShared;
									break;
								case QueryClient.SCOPE_PROJECT_AREA:
									var name = this.self.messages.labelPredefined;
									break;
								default:
									var name = result[i].scopeName;
									break;
							}
							subWidget.populate(this.self.navSection.addSubSection(subWidget, name, null, null, !(result[i].scope == QueryClient.SCOPE_PROJECT_AREA)));
							this.self._sharedSections[result[i].scopeItemId] = subWidget;
						}
						this.self._sharedSections[result[i].scopeItemId]._loadSavedQueries(result[i].queries);
					}
					this.self.parentController.updateWelcomePageQueries(result);
				},
				failure: function(error) {
					this.parentController.handleError(error, this.self.messages.errorFetchingSharedQueries);
				}
			};
			var responseHandler= new ServiceResponseHandler(serviceObject, "success", "failure");
			QueryClient.getScopedQueries(responseHandler, QueryClient.SCOPE_OTHER_CONTRIBUTORS, getAuthenticatedContributor().itemId, false, true);
		},
		
		_loadSavedQueries: function(queries) {
			if (this._queryLinks) {
				for (var i = 0; i < this._queryLinks.length; i++) {
					if (this._queryLinks[i].parentNode)
						this._queryLinks[i].parentNode.removeChild(this._queryLinks[i]);
				}
			}
				
			this._queryLinks = new Array();
			this._hidePartialLinks = new Array();
			
			// if we have no queries, hide ourself
			if (!queries || queries.length === 0) {
				dojo.addClass(this.navSection.domNode,"net-jazz-ajax-hidden");
				return; // early return
			}
			dojo.removeClass(this.navSection.domNode,"net-jazz-ajax-hidden");
			// otherwise, populate the full and partial view caches
			for (var i=0; i<queries.length; i++) {
				// always add the query item to the full view list
				this._queryLinks[i] = createRow(queries[i], this.navSection, this.messages);
				// only add to the partial list view if we're under our count
				if(i >= QUERY_PARTIAL_VIEW_COUNT) {
					this._hidePartialLinks.push(this._queryLinks[i]);
					dojo.addClass(this._queryLinks[i],"net-jazz-ajax-hidden");
					
				}
			}
			this.navSection.update();
		},
		
		hasShowPartial: function() {
			return this._hidePartialLinks ? this._hidePartialLinks.length > 0 : false;
		},
		
		onShowPartial: function() {
			if (this._hidePartialLinks)
				for (var i = 0; i < this._hidePartialLinks.length; i++)
					dojo.addClass(this._hidePartialLinks[i],"net-jazz-ajax-hidden");
		},
		
		onMaximize: function() {
			if (this._hidePartialLinks)
				for (var i = 0; i < this._hidePartialLinks.length; i++)
					dojo.removeClass(this._hidePartialLinks[i],"net-jazz-ajax-hidden");
		},
		
		_loadSavedQueriesError: function(errorObj) {
			this.parentController.handleError(errorObj, this.messages.errorFetchingSharedQueries);
		}
	}
);

var SavedQueries = com.ibm.team.workitem.web.ui.internal.view.query.SavedQueries;

})();