/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectAttributeDialog");

dojo.require("net.jazz.ajax.ui.internal.Dialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {
	
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient= com.ibm.team.workitem.web.client.internal.QueryClient;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectAttributeDialog", null, {
	
	constructor: function(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		if (!dojo.isFunction(params.onOk)) {
			throw new Error("Missing function for 'onOk' parameter");
		}
		this._multiSelect= params.multiSelect == null || params.multiSelect; // null == true
		var widgetParams= {
			primaryTitle: (params.title ? params.title : this._multiSelect ? this.messages.dialogTitleSelectAttributes : this.messages.dialogTitleSelectAttribute)
		};
		this._onOk= params.onOk;
		this._filterAttribute= params.filterAttribute;
		this._attributeTree= params.attributeTree; 

		var self= this;
		var onOk= function(attributes) {
			self._onOk(attributes);
			onCancel();
		};
		var onCancel= function() {
			self.floatingPane.close();
		};
		
		this.contentWidget= new com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectAttributeDialogContent({
			onOk: onOk, 
			onCancel: onCancel, 
			multiSelect: this._multiSelect, 
			filterAttribute: this._filterAttribute,
			attributeTree: this._attributeTree
		});
		widgetParams.contentNode = this.contentWidget.domNode;
		this.floatingPane= new net.jazz.ajax.ui.internal.Dialog( widgetParams );
		this.contentWidget.init();
	}
	
});

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectAttributeDialogContent",
	[dijit._Widget, dijit._Templated], {
	
	templateString: '<div style="padding: 1em 1em 1em 1em;">' +
						'<div width="100%"><select dojoAttachPoint="_select" size="20" style="width: 100%;"></select></div>' +
						'<div style="padding: 1em 0em; float: right;">' +
							'<button dojoAttachPoint="_okCmd" disabled="disabled">${textButtonOK}</button>' +
							'<button dojoAttachPoint="_cancelCmd">${textButtonCancel}</button>' +
						'</div>' +
					' </div>',
	
	constructor: function(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.textButtonOK = this.messages.buttonOK;
		this.textButtonCancel = this.messages.buttonCancel;
		this._onOk= params.onOk;
		this._onCancel= params.onCancel;
		this._multiSelect= params.multiSelect;
		this._attributeTree= params.attributeTree;
		this._filterAttribute= params.filterAttribute;
	},
	
	postCreate: function() {
		this.connect(this._okCmd, "onclick", this._doOnOk);
		this.connect(this._cancelCmd, "onclick", this._onCancel);
		this.connect(this._select, "onchange", this._onChange);
		
		if (this._multiSelect) {
			this._select.multiple= "multiple";
		}
	},
	
	init: function() {
		if (!this._attributeTree) {
			var srh= new ServiceResponseHandler(this, "_setAttributeTree", "_handleAttributeError");
			QueryClient.getAttributeTree(srh);
		} else {
			this._setAttributeTree(this._attributeTree);
		}
	},
	
	_setAttributeTree: function(attributeTree) {
		
		var attributeMap= new Object();
		var filteredAttributes= new Array();
		var attributes= attributeTree.children;
		
		var filter= this._filterAttribute;
		if (filter) {
			for (var i= 0; i < attributes.length; i++) {
				var item= attributes[i].item;
				if (item[filter]) 
					filteredAttributes.push(item);
			}
		} else {
			for (var i= 0; i < attributes.length; i++) {
				filteredAttributes.push(attributes[i].item);
			}
		}
		filteredAttributes.sort(this._sortAttributes);
		
		for (var k= 0; k < filteredAttributes.length; k++) {
			var item= filteredAttributes[k];
			var option= new Option(item.label, item.id, false, false);
			this._select[k]= option;
			attributeMap[item.id]= item;
		}
		this._attributeMap= attributeMap;
	},
	
	_doOnOk: function() {
		var selection= new Array();
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].selected) {
				var item= this._attributeMap[select[i].value];
				selection.push(item);
			}
		}
		this._onOk(selection);
	},
	
	_onChange: function() {
		this._okCmd.disabled= !this._hasSelection();
	},
	
	_hasSelection: function() {
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].selected) {
				return true;
			}
		}
		return false;
	},
	
	_sortAttributes: function(entry1, entry2){
		if (entry1.label < entry2.label) {
			return -1;
		}
		else if (entry1.label > entry2.label) {
			return 1;
		}
		return 0;
	},
	
	_handleAttributeError: function(error) {
		console.error(error);
	}
	
});

})();
